@@ -1,15 +1,6 @@
 /*
- * Copyright (C) 2022 by Claudio Cambra <claudio.cambra@nextcloud.com>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- * for more details.
+ * SPDX-FileCopyrightText: 2022 Nextcloud GmbH and Nextcloud contributors
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "sharemodel.h"
@@ -220,11 +211,21 @@ void ShareModel::resetData()
     _fetchOngoing = false;
     _hasInitialShareFetchCompleted = false;
     _sharees.clear();
+    _displayShareOwner = false;
+    _shareOwnerDisplayName.clear();
+    _shareOwnerAvatar.clear();
+    _sharedWithMeExpires = false;
+    _sharedWithMeRemainingTimeString.clear();
 
     Q_EMIT sharePermissionsChanged();
     Q_EMIT fetchOngoingChanged();
     Q_EMIT hasInitialShareFetchCompletedChanged();
     Q_EMIT shareesChanged();
+    Q_EMIT displayShareOwnerChanged();
+    Q_EMIT shareOwnerDisplayNameChanged();
+    Q_EMIT shareOwnerAvatarChanged();
+    Q_EMIT sharedWithMeExpiresChanged();
+    Q_EMIT sharedWithMeRemainingTimeStringChanged();
 
     endResetModel();
 }
@@ -298,6 +299,7 @@ void ShareModel::updateData()
     _placeholderLinkShare.reset(new Share(_accountState->account(),
                                           placeholderLinkShareId,
                                           _accountState->account()->id(),
+                                          _accountState->account()->id(),
                                           _accountState->account()->davDisplayName(),
                                           _sharePath,
                                           Share::TypePlaceholderLink));
@@ -305,6 +307,7 @@ void ShareModel::updateData()
     _internalLinkShare.reset(new Share(_accountState->account(),
                                        internalLinkShareId,
                                        _accountState->account()->id(),
+                                       _accountState->account()->id(),
                                        _accountState->account()->davDisplayName(),
                                        _sharePath,
                                        Share::TypeInternalLink));
@@ -312,6 +315,7 @@ void ShareModel::updateData()
     _secureFileDropPlaceholderLinkShare.reset(new Share(_accountState->account(),
                                                         secureFileDropPlaceholderLinkShareId,
                                                         _accountState->account()->id(),
+                                                        _accountState->account()->id(),
                                                         _accountState->account()->davDisplayName(),
                                                         _sharePath,
                                                         Share::TypeSecureFileDropPlaceholderLink));
@@ -319,7 +323,9 @@ void ShareModel::updateData()
     auto job = new PropfindJob(_accountState->account(), _sharePath);
     job->setProperties(QList<QByteArray>() << "http://open-collaboration-services.org/ns:share-permissions"
                                            << "http://owncloud.org/ns:fileid" // numeric file id for fallback private link generation
-                                           << "http://owncloud.org/ns:privatelink");
+                                           << "http://owncloud.org/ns:privatelink"
+                                           << "http://owncloud.org/ns:owner-id"
+                                           << "http://owncloud.org/ns:owner-display-name");
     job->setTimeout(10 * 1000);
     connect(job, &PropfindJob::result, this, &ShareModel::slotPropfindReceived);
     connect(job, &PropfindJob::finishedWithError, this, [&](const QNetworkReply *reply) {
@@ -351,18 +357,24 @@ void ShareModel::initShareManager()
     if (_manager.isNull() && sharingPossible) {
         _manager.reset(new ShareManager(_accountState->account(), this));
         connect(_manager.data(), &ShareManager::sharesFetched, this, &ShareModel::slotSharesFetched);
+        connect(_manager.data(), &ShareManager::sharedWithMeFetched, this, &ShareModel::slotSharedWithMeFetched);
         connect(_manager.data(), &ShareManager::shareCreated, this, [&] {
             _manager->fetchShares(_sharePath);
         });
         connect(_manager.data(), &ShareManager::linkShareCreated, this, &ShareModel::slotAddShare);
         connect(_manager.data(), &ShareManager::linkShareRequiresPassword, this, &ShareModel::requestPasswordForLinkShare);
         connect(_manager.data(), &ShareManager::serverError, this, [this](const int code, const QString &message) {
-            _hasInitialShareFetchCompleted = true;
-            Q_EMIT hasInitialShareFetchCompletedChanged();
+            if (!_hasInitialShareFetchCompleted) {
+                _hasInitialShareFetchCompleted = true;
+                Q_EMIT hasInitialShareFetchCompletedChanged();
+            }
+
+            qCWarning(lcShareModel) << "Error from server from ShareManager class and initShareManager" << code << message;
             emit serverError(code, message);
         });
 
         _manager->fetchShares(_sharePath);
+        _manager->fetchSharedWithMe(_sharePath);
     }
 }
 
@@ -373,7 +385,7 @@ void ShareModel::handlePlaceholderLinkShare()
     auto linkSharePresent = false;
     auto placeholderLinkSharePresent = false;
 
-    for (const auto &share : qAsConst(_shares)) {
+    for (const auto &share : std::as_const(_shares)) {
         const auto shareType = share->getShareType();
 
         if (!linkSharePresent && shareType == Share::TypeLink) {
@@ -403,7 +415,7 @@ void ShareModel::handleSecureFileDropLinkShare()
     auto linkSharePresent = false;
     auto secureFileDropLinkSharePresent = false;
 
-    for (const auto &share : qAsConst(_shares)) {
+    for (const auto &share : std::as_const(_shares)) {
         const auto shareType = share->getShareType();
 
         if (!linkSharePresent && shareType == Share::TypeLink) {
@@ -479,8 +491,7 @@ void ShareModel::slotSharesFetched(const QList<SharePtr> &shares)
     for (const auto &share : shares) {
         if (share.isNull() ||
             share->account().isNull() ||
-            share->getUidOwner() != share->account()->davUser()) {
-
+            share->getUidFileOwner() != share->account()->davUser()) {
             continue;
         }
 
@@ -525,6 +536,47 @@ void ShareModel::slotSharesFetched(const QList<SharePtr> &shares)
     handleLinkShare();
 }
 
+void ShareModel::slotSharedWithMeFetched(const QList<OCC::SharePtr> &shares)
+{
+    qCInfo(lcSharing) << "Fetched" << shares.count() << "shares that have been shared_with_me";
+
+    for (const auto &share : shares) {
+        if (share.isNull()) {
+            continue;
+        }
+
+        const auto selfUserId = share->account()->davUser();
+        if (share->getUidOwner() == selfUserId) {
+            continue;
+        }
+
+        _displayShareOwner = true;
+        Q_EMIT displayShareOwnerChanged();
+        _shareOwnerDisplayName = share->getOwnerDisplayName();
+        Q_EMIT shareOwnerDisplayNameChanged();
+        _shareOwnerAvatar = QStringLiteral("image://avatars/user-id=%1/local-account:%2")
+            .arg(share->getUidOwner(), share->account()->displayName());
+        Q_EMIT shareOwnerAvatarChanged();
+
+        if (share->getShareType() == Share::TypeUser &&
+            share->getShareWith() &&
+            share->getShareWith()->shareWith() == selfUserId)
+        {
+            const auto userShare = share.objectCast<UserGroupShare>();
+            const auto expireDate = userShare->getExpireDate();
+            const auto daysToExpire = QDate::currentDate().daysTo(expireDate);
+            _sharedWithMeExpires = expireDate.isValid();
+            Q_EMIT sharedWithMeExpiresChanged();
+            _sharedWithMeRemainingTimeString = daysToExpire > 1
+                ? tr("%1 days").arg(daysToExpire)
+                :  daysToExpire == 1
+                    ? tr("1 day")
+                    : tr("Today");
+            Q_EMIT sharedWithMeRemainingTimeStringChanged();
+        }
+    }
+}
+
 void ShareModel::setupInternalLinkShare()
 {
     if (!_accountState ||
@@ -597,7 +649,10 @@ void ShareModel::slotAddShare(const SharePtr &share)
     const QPersistentModelIndex sharePersistentIndex(shareModelIndex);
     _shareIdIndexHash.insert(shareId, sharePersistentIndex);
 
-    connect(share.data(), &Share::serverError, this, &ShareModel::slotServerError);
+    connect(share.data(), &Share::serverError, this, [this] (int code, const QString &message) {
+        qCWarning(lcShareModel) << "Error from server from Share class" << code << message;
+        Q_EMIT serverError(code, message);
+    });
     connect(share.data(), &Share::passwordSetError, this, [this, shareId](const int code, const QString &message) {
         _shareIdRecentlySetPasswords.remove(shareId);
         slotSharePasswordSet(shareId);
@@ -620,10 +675,6 @@ void ShareModel::slotAddShare(const SharePtr &share)
         connect(userGroupShare.data(), &UserGroupShare::expireDateSet, this, [this, shareId]{ slotShareExpireDateSet(shareId); });
     }
 
-    if (_manager) {
-        connect(_manager.data(), &ShareManager::serverError, this, &ShareModel::slotServerError);
-    }
-
     handleLinkShare();
     Q_EMIT sharesChanged();
 }
@@ -672,12 +723,6 @@ void ShareModel::slotRemoveShareWithId(const QString &shareId)
     Q_EMIT sharesChanged();
 }
 
-void ShareModel::slotServerError(const int code, const QString &message)
-{
-    qCWarning(lcShareModel) << "Error from server" << code << message;
-    Q_EMIT serverError(code, message);
-}
-
 void ShareModel::slotAddSharee(const ShareePtr &sharee)
 {
     if(!sharee) {
@@ -752,7 +797,7 @@ QString ShareModel::avatarUrlForShare(const SharePtr &share) const
         const auto provider = QStringLiteral("image://tray-image-provider/");
         const auto userId = share->getShareWith()->shareWith();
         const auto avatarUrl = Utility::concatUrlPath(_accountState->account()->url(),
-                                                      QString("remote.php/dav/avatars/%1/%2.png").arg(userId, QString::number(64))).toString();
+                                                      QStringLiteral("remote.php/dav/avatars/%1/%2.png").arg(userId, QString::number(64))).toString();
         return QString(provider + avatarUrl);
     }
 
@@ -1037,26 +1082,6 @@ void ShareModel::toggleShareExpirationDateFromQml(const QVariant &share, const b
     toggleShareExpirationDate(ptr, enable);
 }
 
-void ShareModel::toggleShareNoteToRecipient(const SharePtr &share, const bool enable) const
-{
-    if (share.isNull()) {
-        return;
-    }
-
-    const QString note = enable ? tr("Enter a note for the recipient") : QString();
-    if (const auto linkShare = share.objectCast<LinkShare>()) {
-        linkShare->setNote(note);
-    } else if (const auto userGroupShare = share.objectCast<UserGroupShare>()) {
-        userGroupShare->setNote(note);
-    }
-}
-
-void ShareModel::toggleShareNoteToRecipientFromQml(const QVariant &share, const bool enable) const
-{
-    const auto ptr = share.value<SharePtr>();
-    toggleShareNoteToRecipient(ptr, enable);
-}
-
 void ShareModel::changePermissionModeFromQml(const QVariant &share, const OCC::ShareModel::SharePermissionsMode permissionMode)
 {
     const auto sharePtr = share.value<SharePtr>();
@@ -1379,6 +1404,31 @@ bool ShareModel::isShareDisabledEncryptedFolder() const
     return _isShareDisabledEncryptedFolder;
 }
 
+bool ShareModel::displayShareOwner() const
+{
+    return _displayShareOwner;
+}
+
+QString ShareModel::shareOwnerDisplayName() const
+{
+    return _shareOwnerDisplayName;
+}
+
+QString ShareModel::shareOwnerAvatar() const
+{
+    return _shareOwnerAvatar;
+}
+
+QString ShareModel::sharedWithMeRemainingTimeString() const
+{
+    return _sharedWithMeRemainingTimeString;
+}
+
+bool ShareModel::sharedWithMeExpires() const
+{
+    return _sharedWithMeExpires;
+}
+
 QVariantList ShareModel::sharees() const
 {
     QVariantList returnSharees;
@@ -1403,7 +1453,7 @@ QString ShareModel::generatePassword()
     static const QRegularExpression lowercaseMatch("[a-z]");
     static const QRegularExpression uppercaseMatch("[A-Z]");
     static const QRegularExpression numberMatch("[0-9]");
-    static const QRegularExpression specialCharMatch(QString("[%1]").arg(specialChars.data()));
+    static const QRegularExpression specialCharMatch(QStringLiteral("[%1]").arg(specialChars.data()));
 
     static const std::map<std::string_view, QRegularExpression> matchMap{
         {lowercaseAlphabet, lowercaseMatch},
