@@ -1,15 +1,7 @@
 /*
- * Copyright (C) by Daniel Molkentin <danimo@owncloud.com>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- * for more details.
+ * SPDX-FileCopyrightText: 2018 Nextcloud GmbH and Nextcloud contributors
+ * SPDX-FileCopyrightText: 2014 ownCloud GmbH
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "generalsettings.h"
@@ -21,6 +13,7 @@
 #include "owncloudsetupwizard.h"
 #include "accountmanager.h"
 #include "guiutility.h"
+#include "capabilities.h"
 
 #if defined(BUILD_UPDATER)
 #include "updater/updater.h"
@@ -31,6 +24,11 @@
 #endif
 #endif
 
+#ifdef BUILD_FILE_PROVIDER_MODULE
+#include "macOS/fileprovider.h"
+#include "macOS/fileprovidersettingscontroller.h"
+#endif
+
 #include "ignorelisteditor.h"
 #include "common/utility.h"
 #include "logger.h"
@@ -45,6 +43,7 @@
 #include <QMessageBox>
 
 #include <KZip>
+#include <chrono>
 
 namespace {
 struct ZipEntry {
@@ -124,7 +123,8 @@ bool createDebugArchive(const QString &filename)
         QMessageBox::critical(
             nullptr,
             QObject::tr("Failed to create debug archive"),
-            QObject::tr("Could not create debug archive in selected location!")
+            QObject::tr("Could not create debug archive in selected location!"),
+            QMessageBox::Ok
         );
         return false;
     }
@@ -138,6 +138,27 @@ bool createDebugArchive(const QString &filename)
         zip.addLocalFile(entry.localFilename, entry.zipFilename);
     }
 
+#ifdef BUILD_FILE_PROVIDER_MODULE
+    const auto fileProvider = OCC::Mac::FileProvider::instance();
+    if (fileProvider && fileProvider->fileProviderAvailable()) {
+        const auto tempDir = QTemporaryDir();
+        const auto xpc = fileProvider->xpc();
+        const auto vfsAccounts = OCC::Mac::FileProviderSettingsController::instance()->vfsEnabledAccounts();
+        for (const auto &accountUserIdAtHost : vfsAccounts) {
+            const auto accountState = OCC::AccountManager::instance()->accountFromUserId(accountUserIdAtHost);
+            if (!accountState) {
+                qWarning() << "Could not find account for" << accountUserIdAtHost;
+                continue;
+            }
+            const auto account = accountState->account();
+            const auto vfsLogFilename = QStringLiteral("macOS_vfs_%1.log").arg(account->davUser());
+            const auto vfsLogPath = tempDir.filePath(vfsLogFilename);
+            xpc->createDebugArchiveForExtension(accountUserIdAtHost, vfsLogPath);
+            zip.addLocalFile(vfsLogPath, vfsLogFilename);
+        }
+    }
+#endif
+
     const auto clientParameters = QCoreApplication::arguments().join(' ').toUtf8();
     zip.prepareWriting("__nextcloud_client_parameters.txt", {}, {}, clientParameters.size());
     zip.writeData(clientParameters, clientParameters.size());
@@ -160,10 +181,16 @@ GeneralSettings::GeneralSettings(QWidget *parent)
 {
     _ui->setupUi(this);
 
+    updatePollIntervalVisibility();
+    
     connect(_ui->serverNotificationsCheckBox, &QAbstractButton::toggled,
         this, &GeneralSettings::slotToggleOptionalServerNotifications);
     _ui->serverNotificationsCheckBox->setToolTip(tr("Server notifications that require attention."));
 
+    connect(_ui->chatNotificationsCheckBox, &QAbstractButton::toggled,
+            this, &GeneralSettings::slotToggleChatNotifications);
+    _ui->chatNotificationsCheckBox->setToolTip(tr("Show chat notification dialogs."));
+
     connect(_ui->callNotificationsCheckBox, &QAbstractButton::toggled,
         this, &GeneralSettings::slotToggleCallNotifications);
     _ui->callNotificationsCheckBox->setToolTip(tr("Show call notification dialogs."));
@@ -202,18 +229,14 @@ GeneralSettings::GeneralSettings(QWidget *parent)
 
     // misc
     connect(_ui->monoIconsCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::saveMiscSettings);
-    connect(_ui->crashreporterCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::saveMiscSettings);
     connect(_ui->newFolderLimitCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::saveMiscSettings);
     connect(_ui->newFolderLimitSpinBox, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged), this, &GeneralSettings::saveMiscSettings);
     connect(_ui->existingFolderLimitCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::saveMiscSettings);
     connect(_ui->stopExistingFolderNowBigSyncCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::saveMiscSettings);
     connect(_ui->newExternalStorage, &QAbstractButton::toggled, this, &GeneralSettings::saveMiscSettings);
     connect(_ui->moveFilesToTrashCheckBox, &QAbstractButton::toggled, this, &GeneralSettings::saveMiscSettings);
-
-#ifndef WITH_CRASHREPORTER
-    _ui->crashreporterCheckBox->setVisible(false);
-#endif
-
+    connect(_ui->remotePollIntervalSpinBox, &QSpinBox::valueChanged, this, &GeneralSettings::slotRemotePollIntervalChanged);
+    
     // Hide on non-Windows, or WindowsVersion < 10.
     // The condition should match the default value of ConfigFile::showInExplorerNavigationPane.
 #ifdef Q_OS_WIN
@@ -244,6 +267,10 @@ GeneralSettings::GeneralSettings(QWidget *parent)
     // accountAdded means the wizard was finished and the wizard might change some options.
     connect(AccountManager::instance(), &AccountManager::accountAdded, this, &GeneralSettings::loadMiscSettings);
 
+#if defined(BUILD_UPDATER)
+    loadUpdateChannelsList();
+#endif
+
     customizeStyle();
 }
 
@@ -267,10 +294,11 @@ void GeneralSettings::loadMiscSettings()
 
     _ui->monoIconsCheckBox->setChecked(cfgFile.monoIcons());
     _ui->serverNotificationsCheckBox->setChecked(cfgFile.optionalServerNotifications());
-    _ui->callNotificationsCheckBox->setEnabled(_ui->serverNotificationsCheckBox->isEnabled());
+    _ui->chatNotificationsCheckBox->setEnabled(cfgFile.optionalServerNotifications());
+    _ui->chatNotificationsCheckBox->setChecked(cfgFile.showChatNotifications());
+    _ui->callNotificationsCheckBox->setEnabled(cfgFile.optionalServerNotifications());
     _ui->callNotificationsCheckBox->setChecked(cfgFile.showCallNotifications());
     _ui->showInExplorerNavigationPaneCheckBox->setChecked(cfgFile.showInExplorerNavigationPane());
-    _ui->crashreporterCheckBox->setChecked(cfgFile.crashReporter());
     _ui->newExternalStorage->setChecked(cfgFile.confirmExternalStorage());
     _ui->monoIconsCheckBox->setChecked(cfgFile.monoIcons());
     _ui->moveFilesToTrashCheckBox->setChecked(cfgFile.moveToTrash());
@@ -285,17 +313,38 @@ void GeneralSettings::loadMiscSettings()
     _ui->newExternalStorage->setChecked(cfgFile.confirmExternalStorage());
     _ui->monoIconsCheckBox->setChecked(cfgFile.monoIcons());
 
-#if defined(BUILD_UPDATER)
-    const auto validUpdateChannels = cfgFile.validUpdateChannels();
-    _ui->updateChannel->clear();
-    _ui->updateChannel->addItems(validUpdateChannels);
-    const auto currentUpdateChannelIndex = validUpdateChannels.indexOf(cfgFile.currentUpdateChannel());
-    _ui->updateChannel->setCurrentIndex(currentUpdateChannelIndex != -1? currentUpdateChannelIndex : 0);
-    connect(_ui->updateChannel, &QComboBox::currentTextChanged, this, &GeneralSettings::slotUpdateChannelChanged);
-#endif
+    const auto interval = cfgFile.remotePollInterval(); 
+    _ui->remotePollIntervalSpinBox->setValue(static_cast<int>(interval.count() / 1000));
+    updatePollIntervalVisibility(); 
 }
 
 #if defined(BUILD_UPDATER)
+void GeneralSettings::loadUpdateChannelsList() {
+    ConfigFile cfgFile;
+    if (cfgFile.serverHasValidSubscription()) {
+        _ui->updateChannel->hide();
+        _ui->updateChannelLabel->hide();
+        _ui->restoreUpdateChannelButton->hide();
+        return;
+    }
+
+    const auto validUpdateChannels = cfgFile.validUpdateChannels();
+    const auto currentUpdateChannel = cfgFile.currentUpdateChannel();
+    if (_currentUpdateChannelList.isEmpty() || _currentUpdateChannelList != validUpdateChannels){
+        _currentUpdateChannelList = validUpdateChannels;
+        _ui->updateChannel->clear();
+        _ui->updateChannel->addItems(_currentUpdateChannelList);
+        const auto currentUpdateChannelIndex = _currentUpdateChannelList.indexOf(currentUpdateChannel);
+        _ui->updateChannel->setCurrentIndex(currentUpdateChannelIndex != -1 ? currentUpdateChannelIndex : 0);
+        connect(_ui->updateChannel, &QComboBox::currentTextChanged, this, &GeneralSettings::slotUpdateChannelChanged);
+    }
+
+    const auto defaultUpdateChannel = cfgFile.defaultUpdateChannel();
+    _ui->restoreUpdateChannelButton->setText(tr("Restore to &%1").arg(updateChannelToLocalized(defaultUpdateChannel)));
+    _ui->restoreUpdateChannelButton->setEnabled(currentUpdateChannel != defaultUpdateChannel);
+    connect(_ui->restoreUpdateChannelButton, &QPushButton::clicked, this, &GeneralSettings::slotRestoreUpdateChannel);
+}
+
 void GeneralSettings::slotUpdateInfo()
 {
     ConfigFile config;
@@ -350,28 +399,43 @@ void GeneralSettings::slotUpdateInfo()
 #endif
 }
 
+void GeneralSettings::setAndCheckNewUpdateChannel(const QString &newChannel) {
+    ConfigFile().setUpdateChannel(newChannel);
+    if (auto updater = qobject_cast<OCUpdater *>(Updater::instance())) {
+        updater->setUpdateUrl(Updater::updateUrl());
+        updater->checkForUpdate();
+    }
+#if defined(Q_OS_MAC) && defined(HAVE_SPARKLE)
+    else if (auto updater = qobject_cast<SparkleUpdater *>(Updater::instance())) {
+        updater->setUpdateUrl(Updater::updateUrl());
+        updater->checkForUpdate();
+    }
+#endif
+}
+
+QString GeneralSettings::updateChannelToLocalized(const QString &channel) const
+{
+    if (channel == QStringLiteral("stable")) {
+        return tr("stable");
+    }
+
+    if (channel == QStringLiteral("beta")) {
+        return tr("beta");
+    }
+
+    if (channel == QStringLiteral("daily")) {
+        return tr("daily");
+    }
+
+    if (channel == QStringLiteral("enterprise")) {
+        return tr("enterprise");
+    }
+
+    return QString{};
+}
+
 void GeneralSettings::slotUpdateChannelChanged()
 {
-    const auto updateChannelToLocalized = [](const QString &channel) {
-        if (channel == QStringLiteral("stable")) {
-            return tr("stable");
-        }
-
-        if (channel == QStringLiteral("beta")) {
-            return tr("beta");
-        }
-
-        if (channel == QStringLiteral("daily")) {
-            return tr("daily");
-        }
-
-        if (channel == QStringLiteral("enterprise")) {
-            return tr("enterprise");
-        }
-
-        return QString{};
-    };
-
     const auto updateChannelFromLocalized = [](const int index) {
         switch(index) {
         case 1:
@@ -389,43 +453,46 @@ void GeneralSettings::slotUpdateChannelChanged()
     };
 
     ConfigFile configFile;
-    const auto channel = updateChannelFromLocalized(_ui->updateChannel->currentIndex());
-    if (channel == configFile.currentUpdateChannel()) {
+    const auto newChannel = updateChannelFromLocalized(_ui->updateChannel->currentIndex());
+    const auto currentUpdateChannel = configFile.currentUpdateChannel();
+    if (newChannel == currentUpdateChannel) {
         return;
     }
 
-    const auto enterprise = configFile.validUpdateChannels().contains("enterprise") ? tr("- enterprise: contains stable versions for customers.\n",
-                                                                                         "description of enterprise update channel for enterprise customers")
-                                                                                    : "";
+    if (newChannel == configFile.defaultUpdateChannel()) {
+        restoreUpdateChannel();
+        return;
+    }
+
+    _ui->restoreUpdateChannelButton->setEnabled(true);
+
+    const auto nonEnterpriseOptions = tr("- beta: contains versions with new features that may not be tested thoroughly\n"
+                                    "- daily: contains versions created daily only for testing and development\n"
+                                    "\n"
+                                    "Downgrading versions is not possible immediately: changing from beta to stable means waiting for the new stable version.",
+                                    "list of available update channels to non enterprise users and downgrading warning");
+    const auto enterpriseOptions = tr("- enterprise: contains stable versions for customers.\n"
+                                    "\n"
+                                    "Downgrading versions is not possible immediately: changing from stable to enterprise means waiting for the new enterprise version.",
+                                    "list of available update channels to enterprise users and downgrading warning");
+
     auto msgBox = new QMessageBox(
         QMessageBox::Warning,
         tr("Changing update channel?"),
         tr("The channel determines which upgrades will be offered to install:\n"
-           "- stable: contains tested versions considered reliable\n"
-           "- beta: contains versions with new features that may not be tested thoroughly\n"
-           "- daily: contains versions created daily only for testing and development\n"
-           "%1\n"
-           "Downgrading versions is not possible immediately: changing from beta to stable means waiting for the new stable version.").arg(enterprise),
+           "- stable: contains tested versions considered reliable\n",
+           "starts list of available update channels, stable is always available")
+            .append(configFile.validUpdateChannels().contains("enterprise") ? enterpriseOptions : nonEnterpriseOptions),
         QMessageBox::NoButton,
         this);
     const auto acceptButton = msgBox->addButton(tr("Change update channel"), QMessageBox::AcceptRole);
     msgBox->addButton(tr("Cancel"), QMessageBox::RejectRole);
-    connect(msgBox, &QMessageBox::finished, msgBox, [this, channel, msgBox, acceptButton, updateChannelToLocalized] {
+    connect(msgBox, &QMessageBox::finished, msgBox, [this, newChannel, currentUpdateChannel, msgBox, acceptButton] {
         msgBox->deleteLater();
         if (msgBox->clickedButton() == acceptButton) {
-            ConfigFile().setUpdateChannel(channel);
-            if (auto updater = qobject_cast<OCUpdater *>(Updater::instance())) {
-                updater->setUpdateUrl(Updater::updateUrl());
-                updater->checkForUpdate();
-            }
-#if defined(Q_OS_MAC) && defined(HAVE_SPARKLE)
-            else if (auto updater = qobject_cast<SparkleUpdater *>(Updater::instance())) {
-                updater->setUpdateUrl(Updater::updateUrl());
-                updater->checkForUpdate();
-            }
-#endif
+            setAndCheckNewUpdateChannel(newChannel);
         } else {
-            _ui->updateChannel->setCurrentText(updateChannelToLocalized(ConfigFile().currentUpdateChannel()));
+            _ui->updateChannel->setCurrentText(updateChannelToLocalized(currentUpdateChannel));
         }
     });
     msgBox->open();
@@ -455,6 +522,19 @@ void GeneralSettings::slotToggleAutoUpdateCheck()
     bool isChecked = _ui->autoCheckForUpdatesCheckBox->isChecked();
     cfgFile.setAutoUpdateCheck(isChecked, QString());
 }
+
+void GeneralSettings::restoreUpdateChannel()
+{
+    const auto defaultUpdateChannel = ConfigFile().defaultUpdateChannel();
+    _ui->restoreUpdateChannelButton->setEnabled(false);
+    _ui->updateChannel->setCurrentText(updateChannelToLocalized(defaultUpdateChannel));
+    setAndCheckNewUpdateChannel(defaultUpdateChannel);
+}
+
+void GeneralSettings::slotRestoreUpdateChannel()
+{
+    restoreUpdateChannel();
+}
 #endif // defined(BUILD_UPDATER)
 
 void GeneralSettings::saveMiscSettings()
@@ -472,7 +552,6 @@ void GeneralSettings::saveMiscSettings()
     Theme::instance()->setSystrayUseMonoIcons(useMonoIcons);
 
     cfgFile.setMonoIcons(useMonoIcons);
-    cfgFile.setCrashReporter(_ui->crashreporterCheckBox->isChecked());
     cfgFile.setMoveToTrash(_ui->moveFilesToTrashCheckBox->isChecked());
     cfgFile.setNewBigFolderSizeLimit(newFolderLimitEnabled, _ui->newFolderLimitSpinBox->value());
     cfgFile.setConfirmExternalStorage(_ui->newExternalStorage->isChecked());
@@ -499,9 +578,16 @@ void GeneralSettings::slotToggleOptionalServerNotifications(bool enable)
 {
     ConfigFile cfgFile;
     cfgFile.setOptionalServerNotifications(enable);
+    _ui->chatNotificationsCheckBox->setEnabled(enable);
     _ui->callNotificationsCheckBox->setEnabled(enable);
 }
 
+void GeneralSettings::slotToggleChatNotifications(bool enable)
+{
+    ConfigFile cfgFile;
+    cfgFile.setShowChatNotifications(enable);
+}
+
 void GeneralSettings::slotToggleCallNotifications(bool enable)
 {
     ConfigFile cfgFile;
@@ -512,8 +598,11 @@ void GeneralSettings::slotShowInExplorerNavigationPane(bool checked)
 {
     ConfigFile cfgFile;
     cfgFile.setShowInExplorerNavigationPane(checked);
+
+#ifdef Q_OS_WIN
     // Now update the registry with the change.
     FolderMan::instance()->navigationPaneHelper().setShowInExplorerNavigationPane(checked);
+#endif
 }
 
 void GeneralSettings::slotIgnoreFilesEditor()
@@ -545,7 +634,7 @@ void GeneralSettings::slotCreateDebugArchive()
         QMessageBox::information(
             this,
             tr("Debug Archive Created"),
-            tr("Debug archive is created at %1").arg(filename)
+            tr("Redact information deemed sensitive before sharing! Debug archive created at %1").arg(filename)
         );
     }
 }
@@ -580,4 +669,32 @@ void GeneralSettings::customizeStyle()
 #endif
 }
 
+void GeneralSettings::slotRemotePollIntervalChanged(int seconds) 
+{
+    if (_currentlyLoading) {
+        return;
+    }
+
+    ConfigFile cfgFile;
+    std::chrono::milliseconds interval(seconds * 1000);
+    cfgFile.setRemotePollInterval(interval);
+}
+
+void GeneralSettings::updatePollIntervalVisibility() 
+{
+    const auto accounts = AccountManager::instance()->accounts();
+    const auto pushAvailable = std::any_of(accounts.cbegin(), accounts.cend(), [](const AccountStatePtr &accountState) -> bool {
+        if (!accountState) {
+            return false;
+        }
+        const auto accountPtr = accountState->account();
+        if (!accountPtr) {
+            return false;
+        }
+        return accountPtr->capabilities().availablePushNotifications().testFlag(PushNotificationType::Files);
+    });
+
+    _ui->horizontalLayoutWidget_remotePollInterval->setVisible(!pushAvailable);
+}
+
 } // namespace OCC
