@@ -1,15 +1,6 @@
 /*
- * Copyright (C) 2023 by Claudio Cambra <claudio.cambra@nextcloud.com>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- * for more details.
+ * SPDX-FileCopyrightText: 2023 Nextcloud GmbH and Nextcloud contributors
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "fileprovidersettingscontroller.h"
@@ -167,6 +158,7 @@ public:
         qCInfo(lcFileProviderSettingsController) << "Signalling file provider domain" << userIdAtHost;
         NSFileProviderDomain * const domain = FileProviderUtils::domainForIdentifier(userIdAtHost);
         NSFileProviderManager * const manager = [NSFileProviderManager managerForDomain:domain];
+        [domain release];
         [manager signalEnumeratorForContainerItemIdentifier:NSFileProviderRootContainerItemIdentifier
                                           completionHandler:^(NSError *const error) {
             if (error != nil) {
@@ -186,6 +178,7 @@ public:
     }
 
 public slots:
+    // NOTE: This method will release the provided args so make sure to retain them beforehand
     void enumerateMaterialisedFilesForDomainManager(NSFileProviderManager * const managerForDomain,
                                                     NSFileProviderDomain * const domain)
     {
@@ -194,7 +187,6 @@ public slots:
         [enumerator retain];
 
         FileProviderStorageUseEnumerationObserver *const storageUseObserver = [[FileProviderStorageUseEnumerationObserver alloc] init];
-        [storageUseObserver retain];
         storageUseObserver.enumerationFinishedHandler = ^(NSError *const error) {
             qCInfo(lcFileProviderSettingsController) << "Enumeration finished for" << domain.identifier;
             if (error != nil) {
@@ -211,16 +203,17 @@ public slots:
             const auto qDomainIdentifier = QString::fromNSString(domain.identifier);
             QVector<FileProviderItemMetadata> qMaterialisedItems;
             qMaterialisedItems.reserve(items.count);
+            unsigned long long storageUsage = 0;
             for (const id<NSFileProviderItem> item in items) {
                 const auto itemMetadata = FileProviderItemMetadata::fromNSFileProviderItem(item, qDomainIdentifier);
-                const auto storageUsage = _storageUsage.value(qDomainIdentifier) + itemMetadata.documentSize();
+                storageUsage += itemMetadata.documentSize();
                 qCDebug(lcFileProviderSettingsController) << "Adding item" << itemMetadata.identifier()
                                                           << "with size" << itemMetadata.documentSize()
                                                           << "to storage usage for account" << qDomainIdentifier
                                                           << "with total size" << storageUsage;
                 qMaterialisedItems.append(itemMetadata);
-                _storageUsage.insert(qDomainIdentifier, storageUsage);
             }
+            _storageUsage.insert(qDomainIdentifier, storageUsage);
             _materialisedFiles.insert(qDomainIdentifier, qMaterialisedItems);
 
             emit q->localStorageUsageForAccountChanged(qDomainIdentifier);
@@ -228,6 +221,9 @@ public slots:
 
             [storageUseObserver release];
             [enumerator release];
+
+            [managerForDomain release];
+            [domain release];
         };
         [enumerator enumerateItemsForObserver:storageUseObserver startingAtPage:NSFileProviderInitialPageSortedByName];
     }
@@ -283,7 +279,8 @@ private:
                                                                 << ", returning early.";
                     return;
                 }
-
+                [managerForDomain retain];
+                [domain retain];
                 enumerateMaterialisedFilesForDomainManager(managerForDomain, domain);
             }
         }];
@@ -361,50 +358,58 @@ bool FileProviderSettingsController::vfsEnabledForAccount(const QString &userIdA
     return d->vfsEnabledForAccount(userIdAtHost);
 }
 
-void FileProviderSettingsController::setVfsEnabledForAccount(const QString &userIdAtHost, const bool setEnabled)
+void FileProviderSettingsController::setVfsEnabledForAccount(const QString &userIdAtHost, const bool setEnabled, const bool showInformationDialog)
 {
     const auto enabledAccountsAction = d->setVfsEnabledForAccount(userIdAtHost, setEnabled);
     if (enabledAccountsAction == MacImplementation::VfsAccountsAction::VfsAccountsEnabledChanged) {
         emit vfsEnabledAccountsChanged();
+
+        if (setEnabled && showInformationDialog) {
+            QMessageBox::information(nullptr,
+                                     tr("macOS virtual files enabled"),
+                                     tr("Virtual files have been enabled for this account.\n"
+                                        "Files are accessible in Finder via an entry under the \"Locations\" section.\n"
+                                        "Please note that on macOS, virtual and classic sync folders are separate.\n"));
+        }
     }
 }
 
-bool FileProviderSettingsController::fastEnumerationSetForAccount(const QString &userIdAtHost) const
-{
-    const auto xpc = FileProvider::instance()->xpc();
-    if (!xpc) {
-        return false;
-    }
-    if (const auto state = xpc->fastEnumerationStateForExtension(userIdAtHost)) {
-        return state->second;
-    }
-    return false;
-}
-
-bool FileProviderSettingsController::fastEnumerationEnabledForAccount(const QString &userIdAtHost) const
+bool FileProviderSettingsController::trashDeletionEnabledForAccount(const QString &userIdAtHost) const
 {
     const auto xpc = FileProvider::instance()->xpc();
     if (!xpc) {
         return true;
     }
-    if (const auto fastEnumerationState = xpc->fastEnumerationStateForExtension(userIdAtHost)) {
-        return fastEnumerationState->first;
+    if (const auto trashDeletionState = xpc->trashDeletionEnabledStateForExtension(userIdAtHost)) {
+        return trashDeletionState->first;
     }
     return true;
 }
 
-void FileProviderSettingsController::setFastEnumerationEnabledForAccount(const QString &userIdAtHost, const bool setEnabled)
+bool FileProviderSettingsController::trashDeletionSetForAccount(const QString &userIdAtHost) const
+{
+    const auto xpc = FileProvider::instance()->xpc();
+    if (!xpc) {
+        return false;
+    }
+    if (const auto state = xpc->trashDeletionEnabledStateForExtension(userIdAtHost)) {
+        return state->second;
+    }
+    return false;
+}
+
+void FileProviderSettingsController::setTrashDeletionEnabledForAccount(const QString &userIdAtHost, const bool setEnabled)
 {
     const auto xpc = FileProvider::instance()->xpc();
     if (!xpc) {
         // Reset state of UI elements
-        emit fastEnumerationEnabledForAccountChanged(userIdAtHost);
-        emit fastEnumerationSetForAccountChanged(userIdAtHost);
+        emit trashDeletionEnabledForAccountChanged(userIdAtHost);
+        emit trashDeletionSetForAccountChanged(userIdAtHost);
         return;
     }
-    xpc->setFastEnumerationEnabledForExtension(userIdAtHost, setEnabled);
-    emit fastEnumerationEnabledForAccountChanged(userIdAtHost);
-    emit fastEnumerationSetForAccountChanged(userIdAtHost);
+    xpc->setTrashDeletionEnabledForExtension(userIdAtHost, setEnabled);
+    emit trashDeletionEnabledForAccountChanged(userIdAtHost);
+    emit trashDeletionSetForAccountChanged(userIdAtHost);
 }
 
 unsigned long long FileProviderSettingsController::localStorageUsageForAccount(const QString &userIdAtHost) const
@@ -505,6 +510,60 @@ FileProviderDomainSyncStatus *FileProviderSettingsController::domainSyncStatusFo
     return d->domainSyncStatusForAccount(userIdAtHost);
 }
 
+void FileProviderSettingsController::resetVfsForAccount(const QString &userIdAtHost)
+{
+    qCInfo(lcFileProviderSettingsController) << "Resetting virtual files environment for account" << userIdAtHost;
+    setVfsEnabledForAccount(userIdAtHost, false);
+
+    const auto accountState = AccountManager::instance()->accountFromUserId(userIdAtHost);
+    if (!accountState) {
+        qCWarning(lcFileProviderSettingsController) << "Could not find account for userIdAtHost" << userIdAtHost
+                                                    << "to reset VFS environment.";
+        return;
+    }
+    const auto splitUserId = userIdAtHost.split('@');
+    if (splitUserId.size() != 2) {
+        qCWarning(lcFileProviderSettingsController) << "Invalid userIdAtHost format" << userIdAtHost
+                                                    << "Expected format: userId@host";
+        return;
+    }
+    const auto accUserId = splitUserId.at(0);
+    const auto accHost = splitUserId.at(1);
+
+    // Delete the database in the group container
+    const auto groupContainerPath = FileProviderUtils::groupContainerPath();
+    if (groupContainerPath.isEmpty()) {
+        qCWarning(lcFileProviderSettingsController) << "Could not determine group container path, cannot reset VFS.";
+        return;
+    }
+    const auto dbsPath = QDir::cleanPath(groupContainerPath + "/FileProviderExt/Database");
+    qCInfo(lcFileProviderSettingsController) << "Resetting VFS for account" << userIdAtHost
+                                             << "by deleting database files in" << dbsPath;
+    const auto databases = QDir(dbsPath).entryList(QDir::Files);
+    for (const auto &dbFile : databases) {
+        // Format of db file names is "userId_cloud_nc_com-fileproviderextdatabase.realm"
+        const auto splitDbName = dbFile.split('-');
+        const auto address = splitDbName.at(0).split('_').mid(1).join('.');
+        const auto userId = splitDbName.at(0).split('_').first();
+
+        if (userId != accUserId || address != accHost) {
+            qCInfo(lcFileProviderSettingsController) << "Skipping database file" << dbFile
+                                                     << "for userId" << userId
+                                                     << "and host" << address
+                                                     << "as it does not match the account we are resetting.";
+            continue; // Not the database we are looking for
+        }
+        
+        const auto dbPath = QDir(dbsPath).filePath(dbFile);
+        qCInfo(lcFileProviderSettingsController) << "Deleting database file" << dbPath;
+        if (QFile::exists(dbPath)) {
+            QFile::remove(dbPath);
+        }
+    }
+
+    setVfsEnabledForAccount(userIdAtHost, true);
+}
+
 } // namespace Mac
 
 } // namespace OCC
