@@ -1,25 +1,9 @@
 /*
- * Copyright (C) by Klaas Freitag <freitag@owncloud.com>
- * Copyright (C) by Krzesimir Nowak <krzesimir@endocode.com>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- * for more details.
+ * SPDX-FileCopyrightText: 2018 Nextcloud GmbH and Nextcloud contributors
+ * SPDX-FileCopyrightText: 2012 ownCloud GmbH
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
-#include <QAbstractButton>
-#include <QtCore>
-#include <QProcess>
-#include <QMessageBox>
-#include <QDesktopServices>
-#include <QApplication>
-
 #include "accessmanager.h"
 #include "account.h"
 #include "accountmanager.h"
@@ -31,6 +15,7 @@
 #include "networkjobs.h"
 #include "owncloudgui.h"
 #include "owncloudsetupwizard.h"
+#include "owncloudpropagator_p.h"
 #include "sslerrordialog.h"
 #include "wizard/owncloudwizard.h"
 #include "wizard/owncloudwizardcommon.h"
@@ -39,6 +24,18 @@
 #include "creds/abstractcredentials.h"
 #include "creds/dummycredentials.h"
 
+#ifdef BUILD_FILE_PROVIDER_MODULE
+#include "gui/macOS/fileprovider.h"
+#include "gui/macOS/fileprovidersettingscontroller.h"
+#endif
+
+#include <QAbstractButton>
+#include <QtCore>
+#include <QProcess>
+#include <QMessageBox>
+#include <QDesktopServices>
+#include <QApplication>
+
 namespace OCC {
 
 OwncloudSetupWizard::OwncloudSetupWizard(QObject *parent)
@@ -66,7 +63,7 @@ OwncloudSetupWizard::~OwncloudSetupWizard()
     _ocWizard->deleteLater();
 }
 
-static QPointer<OwncloudSetupWizard> wiz = nullptr;
+static QPointer<OwncloudSetupWizard> owncloudSetupWizard = nullptr;
 
 void OwncloudSetupWizard::runWizard(QObject *obj, const char *amember, QWidget *parent)
 {
@@ -78,24 +75,26 @@ void OwncloudSetupWizard::runWizard(QObject *obj, const char *amember, QWidget *
 
         Theme::instance()->setStartLoginFlowAutomatically(true);
     }
-    if (!wiz.isNull()) {
+    if (!owncloudSetupWizard.isNull()) {
         bringWizardToFrontIfVisible();
         return;
     }
 
-    wiz = new OwncloudSetupWizard(parent);
-    connect(wiz, SIGNAL(ownCloudWizardDone(int)), obj, amember);
+    owncloudSetupWizard = new OwncloudSetupWizard(parent);
+    connect(owncloudSetupWizard, SIGNAL(ownCloudWizardDone(int)), obj, amember);
+    connect(owncloudSetupWizard->_ocWizard, &OwncloudWizard::wizardClosed, obj, [] { owncloudSetupWizard.clear(); });
+
     FolderMan::instance()->setSyncEnabled(false);
-    wiz->startWizard();
+    owncloudSetupWizard->startWizard();
 }
 
 bool OwncloudSetupWizard::bringWizardToFrontIfVisible()
 {
-    if (wiz.isNull()) {
+    if (owncloudSetupWizard.isNull()) {
         return false;
     }
 
-    ownCloudGui::raiseDialog(wiz->_ocWizard);
+    ownCloudGui::raiseDialog(owncloudSetupWizard->_ocWizard);
     return true;
 }
 
@@ -147,20 +146,34 @@ void OwncloudSetupWizard::startWizard()
 }
 
 // also checks if an installation is valid and determines auth type in a second step
-void OwncloudSetupWizard::slotCheckServer(const QString &urlString)
+void OwncloudSetupWizard::slotCheckServer(const QUrl &serverURL, const OCC::WizardProxySettingsDialog::WizardProxySettings &proxySettings)
 {
-    QString fixedUrl = urlString;
-    QUrl url = QUrl::fromUserInput(fixedUrl);
-    // fromUserInput defaults to http, not http if no scheme is specified
-    if (!fixedUrl.startsWith("http://") && !fixedUrl.startsWith("https://")) {
-        url.setScheme("https");
-    }
     AccountPtr account = _ocWizard->account();
-    account->setUrl(url);
+    account->setUrl(serverURL);
 
-    // Reset the proxy which might had been determined previously in ConnectionValidator::checkServerAndAuth()
-    // when there was a previous account.
-    account->networkAccessManager()->setProxy(QNetworkProxy(QNetworkProxy::NoProxy));
+    account->setProxyType(proxySettings._proxyType);
+    switch (proxySettings._proxyType)
+    {
+    case QNetworkProxy::HttpCachingProxy:
+    case QNetworkProxy::FtpCachingProxy:
+    case QNetworkProxy::NoProxy:
+    case QNetworkProxy::ProxyType::DefaultProxy:
+        // Reset the proxy which might had been determined previously in ConnectionValidator::checkServerAndAuth()
+        // when there was a previous account.
+        account->networkAccessManager()->setProxy({QNetworkProxy::NoProxy});
+        break;
+    case QNetworkProxy::Socks5Proxy:
+    case QNetworkProxy::HttpProxy:
+        account->setProxyHostName(proxySettings._host);
+        account->setProxyPort(proxySettings._port);
+        account->setProxyNeedsAuth(proxySettings._needsAuth == WizardProxySettingsDialog::ProxyAuthentication::AuthenticationRequired);
+        if (account->proxyNeedsAuth()) {
+            account->setProxyUser(proxySettings._user);
+            account->setProxyPassword(proxySettings._password);
+        }
+
+        break;
+    }
 
     // And also reset the QSslConfiguration, for the same reason (#6832)
     // Here the client certificate is added, if any. Later it'll be in HttpCredentials
@@ -180,8 +193,7 @@ void OwncloudSetupWizard::slotCheckServer(const QString &urlString)
     account->networkAccessManager()->clearAccessCache();
 
     // Lookup system proxy in a thread https://github.com/owncloud/client/issues/2993
-    if ((ClientProxy::isUsingSystemDefault() && account->networkProxySetting() == Account::AccountNetworkProxySetting::GlobalProxy)
-        || account->proxyType() == QNetworkProxy::DefaultProxy) {
+    if (ClientProxy::isUsingSystemDefault() || account->proxyType() == QNetworkProxy::DefaultProxy) {
         qCDebug(lcWizard) << "Trying to look up system proxy";
         ClientProxy::lookupSystemProxyAsync(account->url(), this, SLOT(slotSystemProxyLookupDone(QNetworkProxy)));
     } else {
@@ -337,6 +349,7 @@ void OwncloudSetupWizard::slotConnectToOCUrl(const QString &url)
     AbstractCredentials *creds = _ocWizard->getCredentials();
     if (creds) {
         _ocWizard->account()->setCredentials(creds);
+        creds->persist();
     }
 
     const auto fetchUserNameJob = new JsonApiJob(_ocWizard->account()->sharedFromThis(), QStringLiteral("/ocs/v1.php/cloud/user"));
@@ -355,8 +368,7 @@ void OwncloudSetupWizard::slotConnectToOCUrl(const QString &url)
 
         _ocWizard->setField(QLatin1String("OCUrl"), url);
         _ocWizard->appendToConfigurationLog(tr("Trying to connect to %1 at %2 …")
-                                                .arg(Theme::instance()->appNameGUI())
-                                                .arg(url));
+                                                .arg(Theme::instance()->appNameGUI(), url));
 
         testOwnCloudConnect();
     });
@@ -374,7 +386,22 @@ void OwncloudSetupWizard::testOwnCloudConnect()
     job->setFollowRedirects(false);
     job->setProperties(QList<QByteArray>() << "getlastmodified");
     connect(job, &PropfindJob::result, _ocWizard, &OwncloudWizard::successfulStep);
-    connect(job, &PropfindJob::finishedWithError, this, &OwncloudSetupWizard::slotAuthError);
+    connect(job, &PropfindJob::finishedWithError, this, [this] (QNetworkReply *reply) -> void {
+        if (reply && reply->error() == QNetworkReply::ContentAccessDenied) {
+            // A 403 might indicate that the terms of service need to be signed.
+            // catch this special case here, fall back to the standard error handler if it's not TOS-related
+            auto davException = OCC::getExceptionFromReply(reply);
+            if (!davException.first.isEmpty() && davException.first == QStringLiteral(R"(OCA\TermsOfService\TermsNotSignedException)")) {
+                // authentication was successful, but the user hasn't signed the terms of service yet.  Prompt for that in the next step
+                qCInfo(lcWizard) << "Terms of service not accepted yet!  Will prompt the user in the next step";
+                _ocWizard->_needsToAcceptTermsOfService = true;
+                _ocWizard->successfulStep();
+                return;
+            }
+        }
+        slotAuthError();
+    });
+
     job->start();
 }
 
@@ -411,14 +438,15 @@ void OwncloudSetupWizard::slotAuthError()
                       "\"%1\". The URL is bad, the server is misconfigured.")
                        .arg(Utility::escape(redirectUrl.toString()));
 
+    } else if (reply->error() == QNetworkReply::ContentNotFoundError) {
         // A 404 is actually a success: we were authorized to know that the folder does
         // not exist. It will be created later...
-    } else if (reply->error() == QNetworkReply::ContentNotFoundError) {
         _ocWizard->successfulStep();
         return;
 
-        // Provide messages for other errors, such as invalid credentials.
     } else if (reply->error() != QNetworkReply::NoError) {
+        // Provide messages for other errors, such as invalid credentials.
+
         if (!_ocWizard->account()->credentials()->stillValid(reply)) {
             errorMsg = tr("Access forbidden by server. To verify that you have proper access, "
                           "<a href=\"%1\">click here</a> to access the service with your browser.")
@@ -427,8 +455,8 @@ void OwncloudSetupWizard::slotAuthError()
             errorMsg = job->errorStringParsingBody();
         }
 
-        // Something else went wrong, maybe the response was 200 but with invalid data.
     } else {
+        // Something else went wrong, maybe the response was 200 but with invalid data.
         errorMsg = tr("There was an invalid response to an authenticated WebDAV request");
     }
 
@@ -465,6 +493,14 @@ bool OwncloudSetupWizard::checkDowngradeAdvised(QNetworkReply *reply)
 
 void OwncloudSetupWizard::slotCreateLocalAndRemoteFolders(const QString &localFolder, const QString &remoteFolder)
 {
+#ifdef BUILD_FILE_PROVIDER_MODULE
+    if (Mac::FileProvider::fileProviderAvailable() && _ocWizard->useVirtualFileSync()) {
+        qCInfo(lcWizard) << "Not creating local/remote folders as because macOS File Provider uses its own sync root";
+        finalizeSetup(true);
+        return;
+    }
+#endif
+
     qCInfo(lcWizard) << "Setup local sync folder for new oC connection " << localFolder;
     const QDir fi(localFolder);
 
@@ -579,7 +615,7 @@ void OwncloudSetupWizard::createRemoteFolder()
 void OwncloudSetupWizard::slotCreateRemoteFolderFinished(QNetworkReply *reply)
 {
     auto error = reply->error();
-    qCDebug(lcWizard) << "** webdav mkdir request finished " << error;
+    qCWarning(lcWizard) << "** webdav mkdir request finished " << error;
     //    disconnect(ownCloudInfo::instance(), SIGNAL(webdavColCreated(QNetworkReply::NetworkError)),
     //               this, SLOT(slotCreateRemoteFolderFinished(QNetworkReply::NetworkError)));
 
@@ -666,6 +702,24 @@ void OwncloudSetupWizard::slotAssistantFinished(int result)
         // is changed.
         auto account = applyAccountChanges();
 
+#ifdef BUILD_FILE_PROVIDER_MODULE
+        if (Mac::FileProvider::fileProviderAvailable() && _ocWizard->useVirtualFileSync()) {
+            Mac::FileProvider::instance()->domainManager()->addFileProviderDomainForAccount(account);
+            // let the user settings know that VFS is enabled
+            Mac::FileProviderSettingsController::instance()->setVfsEnabledForAccount(
+                Mac::FileProviderDomainManager::fileProviderDomainIdentifierFromAccountState(AccountStatePtr(account)),
+                true,
+                false
+            );
+            _ocWizard->appendToConfigurationLog(
+                tr("<font color=\"green\"><b>File Provider-based account %1 successfully created!</b></font>").arg(account->account()->userIdAtHostWithPort()));
+            _ocWizard->done(result);
+            emit ownCloudWizardDone(result);
+
+            return;
+        }
+#endif
+
         QString localFolder = FolderDefinition::prepareLocalPath(_ocWizard->localFolder());
 
         bool startFromScratch = _ocWizard->field("OCSyncFromScratch").toBool();
@@ -675,11 +729,17 @@ void OwncloudSetupWizard::slotAssistantFinished(int result)
             folderDefinition.localPath = localFolder;
             folderDefinition.targetPath = FolderDefinition::prepareTargetPath(_remoteFolder);
             folderDefinition.ignoreHiddenFiles = folderMan->ignoreHiddenFiles();
+#ifndef BUILD_FILE_PROVIDER_MODULE
             if (_ocWizard->useVirtualFileSync()) {
                 folderDefinition.virtualFilesMode = bestAvailableVfsMode();
             }
-            if (folderMan->navigationPaneHelper().showInExplorerNavigationPane())
+#endif
+
+#ifdef Q_OS_WIN
+            if (folderMan->navigationPaneHelper().showInExplorerNavigationPane()) {
                 folderDefinition.navigationPaneClsid = QUuid::createUuid();
+            }
+#endif
 
             auto f = folderMan->addFolder(account, folderDefinition);
             if (f) {
@@ -699,7 +759,7 @@ void OwncloudSetupWizard::slotAssistantFinished(int result)
     }
 
     // notify others.
-    _ocWizard->done(QWizard::Accepted);
+    _ocWizard->done(result);
     emit ownCloudWizardDone(result);
 }
 
@@ -709,7 +769,10 @@ void OwncloudSetupWizard::slotSkipFolderConfiguration()
 
     disconnect(_ocWizard, &OwncloudWizard::basicSetupFinished,
         this, &OwncloudSetupWizard::slotAssistantFinished);
-    _ocWizard->close();
+
+    _ocWizard->done(QDialog::Rejected);
+
+    // Accept to check connectivity, only skip folder setup
     emit ownCloudWizardDone(QDialog::Accepted);
 }
 
@@ -726,7 +789,7 @@ AccountState *OwncloudSetupWizard::applyAccountChanges()
     auto manager = AccountManager::instance();
 
     auto newState = manager->addAccount(newAccount);
-    manager->save();
+    manager->saveAccount(newAccount);
     return newState;
 }
 
