@@ -1,15 +1,7 @@
 /*
- * Copyright (C) by Cédric Bellegarde <gnumdk@gmail.com>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- * for more details.
+ * SPDX-FileCopyrightText: 2017 Nextcloud GmbH and Nextcloud contributors
+ * SPDX-FileCopyrightText: 2013 ownCloud GmbH
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "accountmanager.h"
@@ -109,6 +101,9 @@ Systray::Systray()
     connect(AccountManager::instance(), &AccountManager::accountAdded,
         this, [this]{ showWindow(WindowPosition::Center); });
 #endif
+
+    connect(FolderMan::instance(), &FolderMan::folderListChanged, this, &Systray::slotSyncFoldersChanged);
+    slotSyncFoldersChanged(FolderMan::instance()->map());
 }
 
 void Systray::create()
@@ -301,20 +296,47 @@ void Systray::createResolveConflictsDialog(const OCC::ActivityList &allConflicts
 
     // This call dialog gets deallocated on close conditions
     // by a call from the QML side to the destroyDialog slot
-    auto dialog = QScopedPointer(conflictsDialog->createWithInitialProperties(initialProperties));
+    auto dialog = std::unique_ptr<QObject>(conflictsDialog->createWithInitialProperties(initialProperties));
     if (!dialog) {
         return;
     }
     dialog->setParent(QGuiApplication::instance());
 
-    auto dialogWindow = qobject_cast<QQuickWindow*>(dialog.data());
+    auto dialogWindow = qobject_cast<QQuickWindow*>(dialog.release());
     if (!dialogWindow) {
         return;
     }
     dialogWindow->show();
     dialogWindow->raise();
     dialogWindow->requestActivate();
-    dialog.take();
+}
+
+void Systray::createEncryptionTokenDiscoveryDialog()
+{
+    if (_encryptionTokenDiscoveryDialog) {
+        return;
+    }
+
+    qCDebug(lcSystray) << "Opening an encryption token discovery dialog...";
+
+    const auto encryptionTokenDiscoveryDialog = new QQmlComponent(_trayEngine.get(), QStringLiteral("qrc:/qml/src/gui/tray/EncryptionTokenDiscoveryDialog.qml"));
+
+    if (encryptionTokenDiscoveryDialog->isError()) {
+        qCWarning(lcSystray) << encryptionTokenDiscoveryDialog->errorString();
+        return;
+    }
+
+    _encryptionTokenDiscoveryDialog = encryptionTokenDiscoveryDialog->createWithInitialProperties(QVariantMap{});
+}
+
+void Systray::destroyEncryptionTokenDiscoveryDialog()
+{
+    if (!_encryptionTokenDiscoveryDialog) {
+        return;
+    }
+    qCDebug(lcSystray) << "Closing an encryption token discovery dialog...";
+    _encryptionTokenDiscoveryDialog->deleteLater();
+    _encryptionTokenDiscoveryDialog = nullptr;
 }
 
 bool Systray::raiseDialogs()
@@ -463,6 +485,14 @@ void Systray::slotPauseAllFolders()
     setPauseOnAllFoldersHelper(true);
 }
 
+void Systray::slotSyncFoldersChanged(const OCC::Folder::Map &folderMap)
+{
+    if (const auto currentAnySyncFolders = !folderMap.isEmpty(); currentAnySyncFolders != _anySyncFolders) {
+        _anySyncFolders = currentAnySyncFolders;
+        emit anySyncFoldersChanged();
+    }
+}
+
 void Systray::setPauseOnAllFoldersHelper(bool pause)
 {
     // For some reason we get the raw pointer from Folder::accountState()
@@ -583,17 +613,29 @@ void Systray::setSyncIsPaused(const bool syncIsPaused)
     emit syncIsPausedChanged();
 }
 
+bool Systray::anySyncFolders() const
+{
+    return _anySyncFolders;
+}
+
 /********************************************************************************************/
 /* Helper functions for cross-platform tray icon position and taskbar orientation detection */
 /********************************************************************************************/
 
 void Systray::positionWindowAtTray(QQuickWindow *window) const
 {
-    if (!useNormalWindow()) {
-        window->setScreen(currentScreen());
-        const auto position = computeWindowPosition(window->width(), window->height());
-        window->setPosition(position);
+    if (useNormalWindow()) {
+        return;
     }
+
+    // need to store the current window size before moving the window to another screen,
+    // otherwise it is being incorrectly resized by the OS or Qt when switching to a screen
+    // with a different DPI setting
+    const auto initialSize = window->size();
+    const auto position = computeWindowPosition(initialSize.width(), initialSize.height());
+    window->setPosition(position);
+    window->setScreen(currentScreen());
+    window->resize(initialSize);
 }
 
 void Systray::positionWindowAtScreenCenter(QQuickWindow *window) const
