@@ -1,3 +1,8 @@
+/*
+ * SPDX-FileCopyrightText: 2021 Nextcloud GmbH and Nextcloud contributors
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ */
+
 #ifndef USERMODEL_H
 #define USERMODEL_H
 
@@ -51,6 +56,7 @@ class User : public QObject
     Q_PROPERTY(QColor headerTextColor READ headerTextColor NOTIFY headerTextColorChanged)
     Q_PROPERTY(QColor accentColor READ accentColor NOTIFY accentColorChanged)
     Q_PROPERTY(bool serverHasUserStatus READ serverHasUserStatus CONSTANT)
+    Q_PROPERTY(UserStatus::OnlineStatus status READ status NOTIFY statusChanged)
     Q_PROPERTY(QUrl statusIcon READ statusIcon NOTIFY statusChanged)
     Q_PROPERTY(QString statusEmoji READ statusEmoji NOTIFY statusChanged)
     Q_PROPERTY(QString statusMessage READ statusMessage NOTIFY statusChanged)
@@ -61,6 +67,7 @@ class User : public QObject
     Q_PROPERTY(QString featuredAppAccessibleName READ featuredAppAccessibleName NOTIFY featuredAppChanged)
     Q_PROPERTY(QString avatar READ avatarUrl NOTIFY avatarChanged)
     Q_PROPERTY(bool isConnected READ isConnected NOTIFY accountStateChanged)
+    Q_PROPERTY(bool needsToSignTermsOfService READ needsToSignTermsOfService NOTIFY accountStateChanged)
     Q_PROPERTY(UnifiedSearchResultsListModel* unifiedSearchResultsListModel READ getUnifiedSearchResultsListModel CONSTANT)
     Q_PROPERTY(QVariantList groupFolders READ groupFolders NOTIFY groupFoldersChanged)
 
@@ -71,6 +78,7 @@ public:
     [[nodiscard]] AccountStatePtr accountState() const;
 
     [[nodiscard]] bool isConnected() const;
+    [[nodiscard]] bool needsToSignTermsOfService() const;
     [[nodiscard]] bool isCurrentUser() const;
     void setCurrentUser(const bool &isCurrent);
     [[nodiscard]] Folder *getFolder() const;
@@ -236,6 +244,7 @@ public:
         NameRole = Qt::UserRole + 1,
         ServerRole,
         ServerHasUserStatusRole,
+        StatusRole,
         StatusIconRole,
         StatusEmojiRole,
         StatusMessageRole,
@@ -276,11 +285,16 @@ private:
     void buildUserList();
 };
 
-class ImageProvider : public QQuickImageProvider
+class ImageProvider : public QQuickAsyncImageProvider
 {
+    Q_OBJECT
+
 public:
-    ImageProvider();
-    QImage requestImage(const QString &id, QSize *size, const QSize &requestedSize) override;
+    ImageProvider() = default;
+    QQuickImageResponse *requestImageResponse(const QString &id, const QSize &requestedSize) override;
+
+private:
+    QThreadPool _pool;
 };
 
 class UserAppsModel : public QAbstractListModel
