@@ -1,20 +1,11 @@
 /*
- * Copyright (C) by Klaas Freitag <freitag@owncloud.com>
- * Copyright (C) by Daniel Molkentin <danimo@owncloud.com>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- * for more details.
+ * SPDX-FileCopyrightText: 2018 Nextcloud GmbH and Nextcloud contributors
+ * SPDX-FileCopyrightText: 2015 ownCloud GmbH
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include <QLoggingCategory>
-#include <QNetworkRequest>
+#include <QHstsPolicy>
 #include <QNetworkAccessManager>
 #include <QNetworkReply>
 #include <QNetworkRequest>
@@ -301,7 +292,6 @@ void AbstractNetworkJob::slotFinished()
 
 QByteArray AbstractNetworkJob::responseTimestamp()
 {
-    ASSERT(!_responseTimestamp.isEmpty());
     return _responseTimestamp;
 }
 
@@ -313,20 +303,28 @@ QByteArray AbstractNetworkJob::requestId()
 QString AbstractNetworkJob::errorString() const
 {
     if (_timedout) {
-        return tr("Connection timed out");
-    } else if (!reply()) {
-        return tr("Unknown error: network reply was deleted");
-    } else if (reply()->hasRawHeader("OC-ErrorString")) {
-        return reply()->rawHeader("OC-ErrorString");
-    } else {
-        return networkReplyErrorString(*reply());
+        return tr("The server took too long to respond. Check your connection and try syncing again. If it still doesn’t work, reach out to your server administrator.");
     }
+
+    if (!reply()) {
+        return tr("An unexpected error occurred. Please try syncing again or contact your server administrator if the issue continues.");
+    }
+
+    if (reply()->hasRawHeader("OC-ErrorString")) {
+        return reply()->rawHeader("OC-ErrorString");
+    }
+
+    if (const auto hstsError = hstsErrorStringFromReply(reply())) {
+        return *hstsError;
+    }
+
+    return networkReplyErrorString(*reply());
 }
 
 QString AbstractNetworkJob::errorStringParsingBody(QByteArray *body)
 {
-    const auto base = errorString();
-    if (base.isEmpty() || !reply()) {
+    const auto errorMessage = errorString();
+    if (errorMessage.isEmpty() || !reply()) {
         return QString();
     }
 
@@ -341,7 +339,7 @@ QString AbstractNetworkJob::errorStringParsingBody(QByteArray *body)
         return extra;
     }
 
-    return base;
+    return errorMessage;
 }
 
 QString AbstractNetworkJob::errorStringParsingBodyException(const QByteArray &body) const
@@ -359,7 +357,7 @@ void AbstractNetworkJob::start()
     _timer.start();
 
     const QUrl url = account()->url();
-    const QString displayUrl = QString("%1://%2%3").arg(url.scheme()).arg(url.host()).arg(url.path());
+    const QString displayUrl = QStringLiteral("%1://%2%3").arg(url.scheme()).arg(url.host()).arg(url.path());
 
     QString parentMetaObjectName = parent() ? parent()->metaObject()->className() : "";
     qCInfo(lcNetworkJob) << metaObject()->className() << "created for" << displayUrl << "+" << path() << parentMetaObjectName;
@@ -468,23 +466,96 @@ QString networkReplyErrorString(const QNetworkReply &reply)
     const auto httpStatus = reply.attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
     const auto httpReason = reply.attribute(QNetworkRequest::HttpReasonPhraseAttribute).toString();
 
-    // Only adjust HTTP error messages of the expected format.
-    if (httpReason.isEmpty() || httpStatus == 0 || !base.contains(httpReason)) {
-        return base;
+    qCWarning(lcNetworkJob) << "Network request error" << base << "HTTP status" << httpStatus << "httpReason" << httpReason;
+
+    QString userFriendlyMessage;
+    switch (httpStatus) {
+        case 400: //Bad Request
+            userFriendlyMessage = QObject::tr("We couldn’t process your request. Please try syncing again later. If this keeps happening, contact your server administrator for help.");
+            break;
+        case 401: //Unauthorized
+            userFriendlyMessage = QObject::tr("You need to sign in to continue. If you have trouble with your credentials, please reach out to your server administrator.");
+            break;
+        case 403: //Forbidden
+            userFriendlyMessage = QObject::tr("You don’t have access to this resource. If you think this is a mistake, please contact your server administrator.");
+            break;
+        case 404: //Not Found
+            userFriendlyMessage = QObject::tr("We couldn’t find what you were looking for. It might have been moved or deleted. If you need help, contact your server administrator.");
+            break;
+        case 407: //Proxy Authentication Required
+            userFriendlyMessage = QObject::tr("It seems you are using a proxy that required authentication. Please check your proxy settings and credentials. If you need help, contact your server administrator.");
+            break;
+        case 408: //Request Timeout
+            userFriendlyMessage = QObject::tr("The request is taking longer than usual. Please try syncing again. If it still doesn’t work, reach out to your server administrator.");
+            break;
+        case 409: //Conflict
+            userFriendlyMessage = QObject::tr("Server files changed while you were working. Please try syncing again. Contact your server administrator if the issue persists.");
+            break;
+        case 410: //Gone
+            userFriendlyMessage = QObject::tr("This folder or file isn’t available anymore. If you need assistance, please contact your server administrator.");
+            break;
+        case 412: //Precondition failed
+            userFriendlyMessage = QObject::tr("The request could not be completed because some required conditions were not met. Please try syncing again later. If you need assistance, please contact your server administrator.");
+            break;
+        case 413: //Payload Too Large
+            userFriendlyMessage = QObject::tr("The file is too big to upload. You might need to choose a smaller file or contact your server administrator for assistance.");
+            break;
+        case 414: //URI Too Long
+            userFriendlyMessage = QObject::tr("The address used to make the request is too long for the server to handle. Please try shortening the information you’re sending or contact your server administrator for assistance.");
+            break;
+        case 415: //Unsupported Media Type
+            userFriendlyMessage = QObject::tr("This file type isn’t supported. Please contact your server administrator for assistance.");
+            break;
+        case 422: //Unprocessable Entity
+            userFriendlyMessage = QObject::tr("The server couldn’t process your request because some information was incorrect or incomplete. Please try syncing again later, or contact your server administrator for assistance.");
+            break;
+        case 423: //Locked
+            userFriendlyMessage = QObject::tr("The resource you are trying to access is currently locked and cannot be modified. Please try changing it later, or contact your server administrator for assistance.");
+            break;
+        case 428: //Precondition Required
+            userFriendlyMessage = QObject::tr("This request could not be completed because it is missing some required conditions. Please try again later, or contact your server administrator for help.");
+            break;
+        case 429: //Too Many Requests
+            userFriendlyMessage = QObject::tr("You made too many requests. Please wait and try again. If you keep seeing this, your server administrator can help.");
+            break;
+        case 500: //Internal Server Error
+            userFriendlyMessage = QObject::tr("Something went wrong on the server. Please try syncing again later, or contact your server administrator if the issue persists.");
+            break;
+        case 501: //Not Implemented
+            userFriendlyMessage = QObject::tr("The server does not recognize the request method. Please contact your server administrator for help.");
+            break;
+        case 502: //Bad Gateway
+            userFriendlyMessage = QObject::tr("We’re having trouble connecting to the server. Please try again soon. If the issue persists, your server administrator can help you.");
+            break;
+        case 503: //Service Unavailable
+            userFriendlyMessage = QObject::tr("The server is busy right now. Please try syncing again in a few minutes or contact your server administrator if it’s urgent.");
+            break;
+        case 504: //Gateway Timeout
+            userFriendlyMessage = QObject::tr("It’s taking too long to connect to the server. Please try again later. If you need help, contact your server administrator.");
+            break;
+        case 505: //HTTP Version Not Supported
+            userFriendlyMessage = QObject::tr("The server does not support the version of the connection being used. Contact your server administrator for help.");
+            break;
+        case 507: //Insufficient Storage
+            userFriendlyMessage = QObject::tr("The server does not have enough space to complete your request. Please check how much quota your user has by contacting your server administrator.");
+            break;
+        case 511: //Network Authentication Required
+            userFriendlyMessage = QObject::tr("Your network needs extra authentication. Please check your connection. Contact your server administrator for help if the issue persists.");
+            break;
+        case 513: //Resource Not Authorized
+            userFriendlyMessage = QObject::tr("You don’t have permission to access this resource. If you believe this is an error, contact your server administrator to ask for assistance.");
+            break;
+        default:
+            userFriendlyMessage = QObject::tr("An unexpected error occurred. Please try syncing again or contact contact your server administrator if the issue continues.");
+            break;
     }
 
-    const auto displayString = reply.request().url().toDisplayString();
-    const auto requestVerb = HttpLogger::requestVerb(reply);
-
-    return AbstractNetworkJob::tr(R"(Server replied "%1 %2" to "%3 %4")").arg(QString::number(httpStatus),
-                                                                                httpReason,
-                                                                                requestVerb,
-                                                                                displayString);
+    return userFriendlyMessage;
 }
 
 void AbstractNetworkJob::retry()
 {
-    ENFORCE(_reply);
+    Q_ASSERT(_reply);
     auto req = _reply->request();
     QUrl requestedUrl = req.url();
     QByteArray verb = HttpLogger::requestVerb(*_reply);
@@ -498,4 +569,29 @@ void AbstractNetworkJob::retry()
     sendRequest(verb, requestedUrl, req, _requestBody);
 }
 
+std::optional<QString> AbstractNetworkJob::hstsErrorStringFromReply(QNetworkReply *reply)
+{
+    if (!reply) {
+        return {};
+    }
+
+    if (reply->error() != QNetworkReply::SslHandshakeFailedError) {
+        return {};
+    }
+
+    if (!(reply->manager() && reply->manager()->isStrictTransportSecurityEnabled())) {
+        return {};
+    }
+
+    const auto host = reply->url().host();
+    const auto policies = reply->manager()->strictTransportSecurityHosts();
+    for (const auto &policy : policies) {
+        if (policy.host() == host && !policy.isExpired()) {
+            return tr("The server enforces strict transport security and does not accept untrusted certificates.");
+        }
+    }
+
+    return {};
+}
+
 } // namespace OCC
