@@ -1,15 +1,7 @@
 /*
- * Copyright (C) by Klaas Freitag <freitag@owncloud.com>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- * for more details.
+ * SPDX-FileCopyrightText: 2018 Nextcloud GmbH and Nextcloud contributors
+ * SPDX-FileCopyrightText: 2014 ownCloud GmbH
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "config.h"
@@ -41,7 +33,7 @@
 #include <QStandardPaths>
 #include <QOperatingSystemVersion>
 
-#define DEFAULT_REMOTE_POLL_INTERVAL 30000 // default remote poll time in milliseconds
+#define DEFAULT_REMOTE_POLL_INTERVAL 30000
 #define DEFAULT_MAX_LOG_LINES 20000
 
 namespace {
@@ -54,7 +46,6 @@ static constexpr char fullLocalDiscoveryIntervalC[] = "fullLocalDiscoveryInterva
 static constexpr char notificationRefreshIntervalC[] = "notificationRefreshInterval";
 static constexpr char monoIconsC[] = "monoIcons";
 static constexpr char deleteFilesThresholdC[] = "deleteFilesThreshold";
-static constexpr char crashReporterC[] = "crashReporter";
 static constexpr char skipUpdateCheckC[] = "skipUpdateCheck";
 static constexpr char autoUpdateCheckC[] = "autoUpdateCheck";
 static constexpr char updateCheckIntervalC[] = "updateCheckInterval";
@@ -83,11 +74,6 @@ static constexpr char proxyUserC[] = "Proxy/user";
 static constexpr char proxyPassC[] = "Proxy/pass";
 static constexpr char proxyNeedsAuthC[] = "Proxy/needsAuth";
 
-static constexpr char useUploadLimitC[] = "BWLimit/useUploadLimit";
-static constexpr char useDownloadLimitC[] = "BWLimit/useDownloadLimit";
-static constexpr char uploadLimitC[] = "BWLimit/uploadLimit";
-static constexpr char downloadLimitC[] = "BWLimit/downloadLimit";
-
 static constexpr char newBigFolderSizeLimitC[] = "newBigFolderSizeLimit";
 static constexpr char useNewBigFolderSizeLimitC[] = "useNewBigFolderSizeLimit";
 static constexpr char notifyExistingFoldersOverLimitC[] = "notifyExistingFoldersOverLimit";
@@ -108,6 +94,8 @@ static const QString defaultUpdateChannelName = "stable";
 static const QStringList enterpriseUpdateChannelsList { QStringLiteral("stable"), QStringLiteral("enterprise") };
 static const QString defaultEnterpriseChannel = "enterprise";
 
+static constexpr char languageC[] = "language";
+
 static constexpr int deleteFilesThresholdDefaultValue = 100;
 }
 
@@ -460,6 +448,16 @@ void OCC::ConfigFile::cleanUpdaterConfiguration()
     settings.sync();
 }
 
+void OCC::ConfigFile::cleanupGlobalNetworkConfiguration()
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+    settings.remove(useUploadLimitC);
+    settings.remove(useDownloadLimitC);
+    settings.remove(uploadLimitC);
+    settings.remove(downloadLimitC);
+    settings.sync();
+}
+
 QString ConfigFile::backup(const QString &fileName) const
 {
     const QString baseFilePath = configPath() + fileName;
@@ -1093,19 +1091,6 @@ void ConfigFile::setMonoIcons(bool useMonoIcons)
     settings.setValue(QLatin1String(monoIconsC), useMonoIcons);
 }
 
-bool ConfigFile::crashReporter() const
-{
-    QSettings settings(configFile(), QSettings::IniFormat);
-    const auto fallback = settings.value(QLatin1String(crashReporterC), true);
-    return getPolicySetting(QLatin1String(crashReporterC), fallback).toBool();
-}
-
-void ConfigFile::setCrashReporter(bool enabled)
-{
-    QSettings settings(configFile(), QSettings::IniFormat);
-    settings.setValue(QLatin1String(crashReporterC), enabled);
-}
-
 bool ConfigFile::automaticLogDir() const
 {
     QSettings settings(configFile(), QSettings::IniFormat);
@@ -1245,6 +1230,17 @@ void ConfigFile::setDesktopEnterpriseChannel(const QString &channel)
     settings.setValue(QLatin1String(desktopEnterpriseChannelName), channel);
 }
 
+QString ConfigFile::language() const
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+    return settings.value(QLatin1String(languageC), QLatin1String("")).toString();
+}
+
+void ConfigFile::setLanguage(const QString& language)
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+    settings.setValue(QLatin1String(languageC), language);
+}
 
 Q_GLOBAL_STATIC(QString, g_configFileName)
 
