@@ -1,25 +1,20 @@
 /*
- * Copyright (C) by Klaas Freitag <freitag@owncloud.com>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- * for more details.
+ * SPDX-FileCopyrightText: 2016 Nextcloud GmbH and Nextcloud contributors
+ * SPDX-FileCopyrightText: 2012 ownCloud GmbH
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #ifndef _THEME_H
 #define _THEME_H
 
+#include "config.h"
+
+#include "syncresult.h"
+
 #include <QIcon>
 #include <QObject>
 #include <QPalette>
 #include <QGuiApplication>
-#include "syncresult.h"
 
 class QString;
 class QObject;
@@ -67,6 +62,7 @@ class OWNCLOUDSYNC_EXPORT Theme : public QObject
 
     Q_PROPERTY(QColor defaultColor READ defaultColor CONSTANT)
 
+    Q_PROPERTY(QVariantMap systemPalette READ systemPalette NOTIFY systemPaletteChanged)
     Q_PROPERTY(bool darkMode READ darkMode NOTIFY darkModeChanged)
 public:
     enum CustomMediaType {
@@ -282,7 +278,10 @@ public:
      *
      * When true, the app always connects to the server directly
      */
-    [[nodiscard]] bool doNotUseProxy() const;
+    [[nodiscard]] constexpr static bool doNotUseProxy()
+    {
+        return DO_NOT_USE_PROXY;
+    }
 
     /**
      * This is only useful when previous version had a different overrideServerUrl
@@ -595,18 +594,28 @@ public:
     [[nodiscard]] bool showVirtualFilesOption() const;
 
     [[nodiscard]] bool enforceVirtualFilesSyncFolder() const;
+    [[nodiscard]] bool disableVirtualFilesSyncFolder() const;
 
     static QColor defaultColor();
 
     static constexpr const char *themePrefix = ":/client/theme/";
 
-    bool darkMode();
+    [[nodiscard]] QVariantMap systemPalette() const;
+    [[nodiscard]] bool darkMode() const;
+
+    /**
+     * Display legacy import dialog
+     *
+     * The user will interact with the dialog to import legacy account when set to true
+     */
+    [[nodiscard]] bool displayLegacyImportDialog() const;
 
 public slots:
     void setOverrideServerUrl(const QString &overrideServerUrl);
     void setForceOverrideServerUrl(bool forceOverride);
     void setVfsEnabled(bool enabled);
     void setStartLoginFlowAutomatically(bool startLoginFlowAuto);
+    void systemPaletteHasChanged();
 
 protected:
 #ifndef TOKEN_AUTH_ONLY
@@ -624,6 +633,7 @@ protected:
 
 signals:
     void systrayUseMonoIconsChanged(bool);
+    void systemPaletteChanged(const QPalette &palette);
     void darkModeChanged();
     void overrideServerUrlChanged();
     void forceOverrideServerUrlChanged();
@@ -634,15 +644,16 @@ private:
     Theme(Theme const &);
     Theme &operator=(Theme const &);
 
+    [[nodiscard]] QString developerStringInfo() const;
+
     void updateMultipleOverrideServers();
-    void connectToPaletteSignal();
+    void connectToPaletteSignal() const;
 #if defined(Q_OS_WIN)
     QPalette reserveDarkPalette; // Windows 11 button and window dark colours
 #endif
 
     static Theme *_instance;
     bool _mono = false;
-    bool _paletteSignalsConnected = false;
 
     QString _overrideServerUrl;
     bool _forceOverrideServerUrl = false;
