@@ -1,23 +1,16 @@
 /*
- * Copyright (C) by Felix Weilbach <felix.weilbach@nextcloud.com>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- * for more details.
+ * SPDX-FileCopyrightText: 2021 Nextcloud GmbH and Nextcloud contributors
+ * SPDX-License-Identifier: GPL-2.0-or-later
  */
 
 #include "userstatusconnector.h"
 #include "userstatusselectormodel.h"
+#include "logger.h"
 
 #include <QTest>
 #include <QSignalSpy>
 #include <QDateTime>
+#include <QStandardPaths>
 
 #include <memory>
 
@@ -69,7 +62,7 @@ public:
         }
     }
 
-    OCC::UserStatus userStatus() const override
+    [[nodiscard]] OCC::UserStatus userStatus() const override
     {
         return {}; // Not implemented
     }
@@ -80,14 +73,14 @@ public:
     }
 
     void setFakePredefinedStatuses(
-        const std::vector<OCC::UserStatus> &statuses)
+        const QVector<OCC::UserStatus> &statuses)
     {
         _predefinedStatuses = statuses;
     }
 
-    OCC::UserStatus userStatusSetByCallerOfSetUserStatus() const { return _userStatusSetByCallerOfSetUserStatus; }
+    [[nodiscard]] OCC::UserStatus userStatusSetByCallerOfSetUserStatus() const { return _userStatusSetByCallerOfSetUserStatus; }
 
-    bool messageCleared() const { return _isMessageCleared; }
+    [[nodiscard]] bool messageCleared() const { return _isMessageCleared; }
 
     void setErrorCouldNotFetchPredefinedUserStatuses(bool value)
     {
@@ -122,7 +115,7 @@ public:
 private:
     OCC::UserStatus _userStatusSetByCallerOfSetUserStatus;
     OCC::UserStatus _userStatus;
-    std::vector<OCC::UserStatus> _predefinedStatuses;
+    QVector<OCC::UserStatus> _predefinedStatuses;
     bool _isMessageCleared = false;
     bool _couldNotFetchPredefinedUserStatuses = false;
     bool _couldNotFetchUserStatus = false;
@@ -137,18 +130,18 @@ class FakeDateTimeProvider : public OCC::DateTimeProvider
 public:
     void setCurrentDateTime(const QDateTime &dateTime) { _dateTime = dateTime; }
 
-    QDateTime currentDateTime() const override { return _dateTime; }
+    [[nodiscard]] QDateTime currentDateTime() const override { return _dateTime; }
 
-    QDate currentDate() const override { return _dateTime.date(); }
+    [[nodiscard]] QDate currentDate() const override { return _dateTime.date(); }
 
 private:
     QDateTime _dateTime;
 };
 
-static std::vector<OCC::UserStatus>
+static QVector<OCC::UserStatus>
 createFakePredefinedStatuses(const QDateTime &currentTime)
 {
-    std::vector<OCC::UserStatus> statuses;
+    QVector<OCC::UserStatus> statuses;
 
     const QString userStatusId("fake-id");
     const QString userStatusMessage("Predefined status");
@@ -161,8 +154,8 @@ createFakePredefinedStatuses(const QDateTime &currentTime)
     clearAt._timestamp = currentTime.addSecs(60 * 60).toSecsSinceEpoch();
     userStatusClearAt = clearAt;
 
-    statuses.emplace_back(userStatusId, userStatusMessage, userStatusIcon,
-        userStatusState, userStatusMessagePredefined, userStatusClearAt);
+    statuses.append({userStatusId, userStatusMessage, userStatusIcon,
+        userStatusState, userStatusMessagePredefined, userStatusClearAt});
 
     return statuses;
 }
@@ -185,9 +178,17 @@ class TestSetUserStatusDialog : public QObject
     Q_OBJECT
 
 private slots:
+    void initTestCase()
+    {
+        OCC::Logger::instance()->setLogFlush(true);
+        OCC::Logger::instance()->setLogDebug(true);
+
+        QStandardPaths::setTestModeEnabled(true);
+    }
+
     void testCtor_fetchStatusAndPredefinedStatuses()
     {
-        const QDateTime currentDateTime(QDateTime::currentDateTime());
+        const QDateTime currentDateTime(QDateTime::currentDateTimeUtc());
 
         const QString userStatusId("fake-id");
         const QString userStatusMessage("Some status");
@@ -218,13 +219,13 @@ private slots:
         QCOMPARE(model.userStatusMessage(), userStatusMessage);
         QCOMPARE(model.userStatusEmoji(), userStatusIcon);
         QCOMPARE(model.onlineStatus(), userStatusState);
-        QCOMPARE(model.clearAt(), tr("1 day"));
+        QCOMPARE(model.clearAtDisplayString(), QStringLiteral("1 day(s)"));
 
         // Were predefined statuses fetched correctly?
-        const auto predefinedStatusesCount = model.predefinedStatusesCount();
+        const auto predefinedStatusesCount = model.predefinedStatuses().count();
         QCOMPARE(predefinedStatusesCount, fakePredefinedStatuses.size());
         for (int i = 0; i < predefinedStatusesCount; ++i) {
-            const auto predefinedStatus = model.predefinedStatus(i);
+            const auto predefinedStatus = model.predefinedStatuses()[i];
             QCOMPARE(predefinedStatus.id(),
                 fakePredefinedStatuses[i].id());
             QCOMPARE(predefinedStatus.message(),
@@ -241,8 +242,8 @@ private slots:
         OCC::UserStatusSelectorModel model(nullptr, nullptr);
 
         QCOMPARE(model.userStatusMessage(), "");
-        QCOMPARE(model.userStatusEmoji(), "😀");
-        QCOMPARE(model.clearAt(), tr("Don't clear"));
+        QCOMPARE(model.userStatusEmoji(), "");
+        QCOMPARE(model.clearAtDisplayString(), QStringLiteral("Don't clear"));
     }
 
     void testCtor_fetchStatusButNoStatusSet_showSensibleDefaults()
@@ -252,23 +253,23 @@ private slots:
             OCC::UserStatus::OnlineStatus::Offline, false, {} });
         OCC::UserStatusSelectorModel model(fakeUserStatusJob);
 
-        QCOMPARE(model.onlineStatus(), OCC::UserStatus::OnlineStatus::Online);
+        QCOMPARE(model.onlineStatus(), OCC::UserStatus::OnlineStatus::Offline);
         QCOMPARE(model.userStatusMessage(), "");
-        QCOMPARE(model.userStatusEmoji(), "😀");
-        QCOMPARE(model.clearAt(), tr("Don't clear"));
+        QCOMPARE(model.userStatusEmoji(), "");
+        QCOMPARE(model.clearAtDisplayString(), QStringLiteral("Don't clear"));
     }
 
-    void testSetOnlineStatus_emitOnlineStatusChanged()
+    void testSetOnlineStatus_emiUserStatusChanged()
     {
         const OCC::UserStatus::OnlineStatus onlineStatus(OCC::UserStatus::OnlineStatus::Invisible);
         auto fakeUserStatusJob = std::make_shared<FakeUserStatusConnector>();
         OCC::UserStatusSelectorModel model(fakeUserStatusJob);
-        QSignalSpy onlineStatusChangedSpy(&model,
-            &OCC::UserStatusSelectorModel::onlineStatusChanged);
+        QSignalSpy userStatusChangedSpy(&model,
+            &OCC::UserStatusSelectorModel::userStatusChanged);
 
         model.setOnlineStatus(onlineStatus);
 
-        QCOMPARE(onlineStatusChangedSpy.count(), 1);
+        QCOMPARE(userStatusChangedSpy.count(), 1);
     }
 
     void testSetUserStatus_setCustomMessage_userStatusSetCorrect()
@@ -284,7 +285,7 @@ private slots:
         model.setOnlineStatus(userStatusState);
         model.setUserStatusMessage(userStatusMessage);
         model.setUserStatusEmoji(userStatusIcon);
-        model.setClearAt(1);
+        model.setClearAt(OCC::UserStatusSelectorModel::ClearStageType::HalfHour);
 
         model.setUserStatus();
         QCOMPARE(finishedSpy.count(), 1);
@@ -305,7 +306,7 @@ private slots:
         auto fakeUserStatusJob = std::make_shared<FakeUserStatusConnector>();
         fakeUserStatusJob->setFakePredefinedStatuses(createFakePredefinedStatuses(createDateTime()));
         OCC::UserStatusSelectorModel model(fakeUserStatusJob);
-        model.setPredefinedStatus(0);
+        model.setPredefinedStatus(model.predefinedStatuses()[0]);
         QSignalSpy finishedSpy(&model, &OCC::UserStatusSelectorModel::finished);
 
         const QString userStatusMessage("Some status");
@@ -313,7 +314,7 @@ private slots:
 
         model.setOnlineStatus(userStatusState);
         model.setUserStatusMessage(userStatusMessage);
-        model.setClearAt(1);
+        model.setClearAt(OCC::UserStatusSelectorModel::ClearStageType::HalfHour);
 
         model.setUserStatus();
         QCOMPARE(finishedSpy.count(), 1);
@@ -333,7 +334,7 @@ private slots:
         auto fakeUserStatusJob = std::make_shared<FakeUserStatusConnector>();
         fakeUserStatusJob->setFakePredefinedStatuses(createFakePredefinedStatuses(createDateTime()));
         OCC::UserStatusSelectorModel model(fakeUserStatusJob);
-        model.setPredefinedStatus(0);
+        model.setPredefinedStatus(model.predefinedStatuses()[0]);
         QSignalSpy finishedSpy(&model, &OCC::UserStatusSelectorModel::finished);
 
         const QString userStatusIcon("❤");
@@ -341,7 +342,7 @@ private slots:
 
         model.setOnlineStatus(userStatusState);
         model.setUserStatusEmoji(userStatusIcon);
-        model.setClearAt(1);
+        model.setClearAt(OCC::UserStatusSelectorModel::ClearStageType::HalfHour);
 
         model.setUserStatus();
         QCOMPARE(finishedSpy.count(), 1);
@@ -369,99 +370,99 @@ private slots:
 
         QSignalSpy userStatusChangedSpy(&model,
             &OCC::UserStatusSelectorModel::userStatusChanged);
-        QSignalSpy clearAtChangedSpy(&model,
-            &OCC::UserStatusSelectorModel::clearAtChanged);
+        QSignalSpy clearAtDisplayStringChangedSpy(&model,
+            &OCC::UserStatusSelectorModel::clearAtDisplayStringChanged);
 
         const auto fakePredefinedUserStatusIndex = 0;
-        model.setPredefinedStatus(fakePredefinedUserStatusIndex);
+        model.setPredefinedStatus(model.predefinedStatuses()[fakePredefinedUserStatusIndex]);
 
         QCOMPARE(userStatusChangedSpy.count(), 1);
-        QCOMPARE(clearAtChangedSpy.count(), 1);
+        QCOMPARE(clearAtDisplayStringChangedSpy.count(), 1);
 
         // Was user status set correctly?
         const auto fakePredefinedUserStatus = fakePredefinedStatuses[fakePredefinedUserStatusIndex];
         QCOMPARE(model.userStatusMessage(), fakePredefinedUserStatus.message());
         QCOMPARE(model.userStatusEmoji(), fakePredefinedUserStatus.icon());
         QCOMPARE(model.onlineStatus(), fakePredefinedUserStatus.state());
-        QCOMPARE(model.clearAt(), tr("1 hour"));
+        QCOMPARE(model.clearAtDisplayString(), QStringLiteral("1 hour(s)"));
     }
 
-    void testSetClear_setClearAtStage0_emitClearAtChangedAndClearAtSet()
+    void testSetClear_setClearAtStage0_emitclearAtDisplayStringChangedAndClearAtSet()
     {
         auto fakeUserStatusJob = std::make_shared<FakeUserStatusConnector>();
         OCC::UserStatusSelectorModel model(fakeUserStatusJob);
-        QSignalSpy clearAtChangedSpy(&model, &OCC::UserStatusSelectorModel::clearAtChanged);
+        QSignalSpy clearAtDisplayStringChangedSpy(&model, &OCC::UserStatusSelectorModel::clearAtDisplayStringChanged);
 
-        const auto clearAtIndex = 0;
-        model.setClearAt(clearAtIndex);
+        const auto clearAtToSet = OCC::UserStatusSelectorModel::ClearStageType::DontClear;
+        model.setClearAt(clearAtToSet);
 
-        QCOMPARE(clearAtChangedSpy.count(), 1);
-        QCOMPARE(model.clearAt(), tr("Don't clear"));
+        QCOMPARE(clearAtDisplayStringChangedSpy.count(), 1);
+        QCOMPARE(model.clearAtDisplayString(), QStringLiteral("Don't clear"));
     }
 
-    void testSetClear_setClearAtStage1_emitClearAtChangedAndClearAtSet()
+    void testSetClear_setClearAtStage1_emitclearAtDisplayStringChangedAndClearAtSet()
     {
         auto fakeUserStatusJob = std::make_shared<FakeUserStatusConnector>();
         OCC::UserStatusSelectorModel model(fakeUserStatusJob);
-        QSignalSpy clearAtChangedSpy(&model, &OCC::UserStatusSelectorModel::clearAtChanged);
+        QSignalSpy clearAtDisplayStringChangedSpy(&model, &OCC::UserStatusSelectorModel::clearAtDisplayStringChanged);
 
-        const auto clearAtIndex = 1;
-        model.setClearAt(clearAtIndex);
+        const auto clearAtToSet = OCC::UserStatusSelectorModel::ClearStageType::HalfHour;
+        model.setClearAt(clearAtToSet);
 
-        QCOMPARE(clearAtChangedSpy.count(), 1);
-        QCOMPARE(model.clearAt(), tr("30 minutes"));
+        QCOMPARE(clearAtDisplayStringChangedSpy.count(), 1);
+        QCOMPARE(model.clearAtDisplayString(), QStringLiteral("30 minute(s)"));
     }
 
-    void testSetClear_setClearAtStage2_emitClearAtChangedAndClearAtSet()
+    void testSetClear_setClearAtStage2_emitclearAtDisplayStringChangedAndClearAtSet()
     {
         auto fakeUserStatusJob = std::make_shared<FakeUserStatusConnector>();
         OCC::UserStatusSelectorModel model(fakeUserStatusJob);
-        QSignalSpy clearAtChangedSpy(&model, &OCC::UserStatusSelectorModel::clearAtChanged);
+        QSignalSpy clearAtDisplayStringChangedSpy(&model, &OCC::UserStatusSelectorModel::clearAtDisplayStringChanged);
 
-        const auto clearAtIndex = 2;
-        model.setClearAt(clearAtIndex);
+        const auto clearAtToSet = OCC::UserStatusSelectorModel::ClearStageType::OneHour;
+        model.setClearAt(clearAtToSet);
 
-        QCOMPARE(clearAtChangedSpy.count(), 1);
-        QCOMPARE(model.clearAt(), tr("1 hour"));
+        QCOMPARE(clearAtDisplayStringChangedSpy.count(), 1);
+        QCOMPARE(model.clearAtDisplayString(), QStringLiteral("1 hour(s)"));
     }
 
-    void testSetClear_setClearAtStage3_emitClearAtChangedAndClearAtSet()
+    void testSetClear_setClearAtStage3_emitclearAtDisplayStringChangedAndClearAtSet()
     {
         auto fakeUserStatusJob = std::make_shared<FakeUserStatusConnector>();
         OCC::UserStatusSelectorModel model(fakeUserStatusJob);
-        QSignalSpy clearAtChangedSpy(&model, &OCC::UserStatusSelectorModel::clearAtChanged);
+        QSignalSpy clearAtDisplayStringChangedSpy(&model, &OCC::UserStatusSelectorModel::clearAtDisplayStringChanged);
 
-        const auto clearAtIndex = 3;
-        model.setClearAt(clearAtIndex);
+        const auto clearAtToSet = OCC::UserStatusSelectorModel::ClearStageType::FourHour;
+        model.setClearAt(clearAtToSet);
 
-        QCOMPARE(clearAtChangedSpy.count(), 1);
-        QCOMPARE(model.clearAt(), tr("4 hours"));
+        QCOMPARE(clearAtDisplayStringChangedSpy.count(), 1);
+        QCOMPARE(model.clearAtDisplayString(), QStringLiteral("4 hour(s)"));
     }
 
-    void testSetClear_setClearAtStage4_emitClearAtChangedAndClearAtSet()
+    void testSetClear_setClearAtStage4_emitclearAtDisplayStringChangedAndClearAtSet()
     {
         auto fakeUserStatusJob = std::make_shared<FakeUserStatusConnector>();
         OCC::UserStatusSelectorModel model(fakeUserStatusJob);
-        QSignalSpy clearAtChangedSpy(&model, &OCC::UserStatusSelectorModel::clearAtChanged);
+        QSignalSpy clearAtDisplayStringChangedSpy(&model, &OCC::UserStatusSelectorModel::clearAtDisplayStringChanged);
 
-        const auto clearAtIndex = 4;
-        model.setClearAt(clearAtIndex);
+        const auto clearAtToSet = OCC::UserStatusSelectorModel::ClearStageType::Today;
+        model.setClearAt(clearAtToSet);
 
-        QCOMPARE(clearAtChangedSpy.count(), 1);
-        QCOMPARE(model.clearAt(), tr("Today"));
+        QCOMPARE(clearAtDisplayStringChangedSpy.count(), 1);
+        QCOMPARE(model.clearAtDisplayString(), QStringLiteral("Today"));
     }
 
-    void testSetClear_setClearAtStage5_emitClearAtChangedAndClearAtSet()
+    void testSetClear_setClearAtStage5_emitclearAtDisplayStringChangedAndClearAtSet()
     {
         auto fakeUserStatusJob = std::make_shared<FakeUserStatusConnector>();
         OCC::UserStatusSelectorModel model(fakeUserStatusJob);
-        QSignalSpy clearAtChangedSpy(&model, &OCC::UserStatusSelectorModel::clearAtChanged);
+        QSignalSpy clearAtDisplayStringChangedSpy(&model, &OCC::UserStatusSelectorModel::clearAtDisplayStringChanged);
 
-        const auto clearAtIndex = 5;
-        model.setClearAt(clearAtIndex);
+        const auto clearAtToSet = OCC::UserStatusSelectorModel::ClearStageType::Week;
+        model.setClearAt(clearAtToSet);
 
-        QCOMPARE(clearAtChangedSpy.count(), 1);
-        QCOMPARE(model.clearAt(), tr("This week"));
+        QCOMPARE(clearAtDisplayStringChangedSpy.count(), 1);
+        QCOMPARE(model.clearAtDisplayString(), QStringLiteral("This week"));
     }
 
     void testClearAtStages()
@@ -469,16 +470,16 @@ private slots:
         auto fakeUserStatusJob = std::make_shared<FakeUserStatusConnector>();
         OCC::UserStatusSelectorModel model(fakeUserStatusJob);
 
-        QCOMPARE(model.clearAt(), tr("Don't clear"));
-        const auto clearAtValues = model.clearAtValues();
-        QCOMPARE(clearAtValues.count(), 6);
+        QCOMPARE(model.clearAtDisplayString(), QStringLiteral("Don't clear"));
+        const auto clearStageTypes = model.clearStageTypes();
+        QCOMPARE(clearStageTypes.count(), 6);
 
-        QCOMPARE(clearAtValues[0], tr("Don't clear"));
-        QCOMPARE(clearAtValues[1], tr("30 minutes"));
-        QCOMPARE(clearAtValues[2], tr("1 hour"));
-        QCOMPARE(clearAtValues[3], tr("4 hours"));
-        QCOMPARE(clearAtValues[4], tr("Today"));
-        QCOMPARE(clearAtValues[5], tr("This week"));
+        QCOMPARE(clearStageTypes[0].value<QVariantMap>()[QStringLiteral("display")], QStringLiteral("Don't clear"));
+        QCOMPARE(clearStageTypes[1].value<QVariantMap>()[QStringLiteral("display")], QStringLiteral("30 minutes"));
+        QCOMPARE(clearStageTypes[2].value<QVariantMap>()[QStringLiteral("display")], QStringLiteral("1 hour"));
+        QCOMPARE(clearStageTypes[3].value<QVariantMap>()[QStringLiteral("display")], QStringLiteral("4 hours"));
+        QCOMPARE(clearStageTypes[4].value<QVariantMap>()[QStringLiteral("display")], QStringLiteral("Today"));
+        QCOMPARE(clearStageTypes[5].value<QVariantMap>()[QStringLiteral("display")], QStringLiteral("This week"));
     }
 
     void testClearAt_clearAtTimestamp()
@@ -496,7 +497,7 @@ private slots:
 
             OCC::UserStatusSelectorModel model(userStatus, std::move(fakeDateTimeProvider));
 
-            QCOMPARE(model.clearAt(), tr("Less than a minute"));
+            QCOMPARE(model.clearAtDisplayString(), QStringLiteral("Less than a minute"));
         }
 
         {
@@ -511,7 +512,7 @@ private slots:
 
             OCC::UserStatusSelectorModel model(userStatus, std::move(fakeDateTimeProvider));
 
-            QCOMPARE(model.clearAt(), tr("1 minute"));
+            QCOMPARE(model.clearAtDisplayString(), QStringLiteral("1 minute(s)"));
         }
 
         {
@@ -526,7 +527,7 @@ private slots:
 
             OCC::UserStatusSelectorModel model(userStatus, std::move(fakeDateTimeProvider));
 
-            QCOMPARE(model.clearAt(), tr("30 minutes"));
+            QCOMPARE(model.clearAtDisplayString(), QStringLiteral("30 minute(s)"));
         }
 
         {
@@ -541,7 +542,7 @@ private slots:
 
             OCC::UserStatusSelectorModel model(userStatus, std::move(fakeDateTimeProvider));
 
-            QCOMPARE(model.clearAt(), tr("1 hour"));
+            QCOMPARE(model.clearAtDisplayString(), QStringLiteral("1 hour(s)"));
         }
 
         {
@@ -556,7 +557,7 @@ private slots:
 
             OCC::UserStatusSelectorModel model(userStatus, std::move(fakeDateTimeProvider));
 
-            QCOMPARE(model.clearAt(), tr("4 hours"));
+            QCOMPARE(model.clearAtDisplayString(), QStringLiteral("4 hour(s)"));
         }
 
         {
@@ -571,7 +572,7 @@ private slots:
 
             OCC::UserStatusSelectorModel model(userStatus, std::move(fakeDateTimeProvider));
 
-            QCOMPARE(model.clearAt(), tr("1 day"));
+            QCOMPARE(model.clearAtDisplayString(), QStringLiteral("1 day(s)"));
         }
 
         {
@@ -586,7 +587,7 @@ private slots:
 
             OCC::UserStatusSelectorModel model(userStatus, std::move(fakeDateTimeProvider));
 
-            QCOMPARE(model.clearAt(), tr("7 days"));
+            QCOMPARE(model.clearAtDisplayString(), QStringLiteral("7 day(s)"));
         }
     }
 
@@ -601,7 +602,7 @@ private slots:
 
             OCC::UserStatusSelectorModel model(userStatus);
 
-            QCOMPARE(model.clearAt(), tr("Today"));
+            QCOMPARE(model.clearAtDisplayString(), QStringLiteral("Today"));
         }
 
         {
@@ -613,7 +614,7 @@ private slots:
 
             OCC::UserStatusSelectorModel model(userStatus);
 
-            QCOMPARE(model.clearAt(), tr("This week"));
+            QCOMPARE(model.clearAtDisplayString(), QStringLiteral("This week"));
         }
     }
 
@@ -628,7 +629,7 @@ private slots:
 
             OCC::UserStatusSelectorModel model(userStatus);
 
-            QCOMPARE(model.clearAt(), tr("30 minutes"));
+            QCOMPARE(model.clearAtDisplayString(), QStringLiteral("30 minute(s)"));
         }
 
         {
@@ -640,7 +641,7 @@ private slots:
 
             OCC::UserStatusSelectorModel model(userStatus);
 
-            QCOMPARE(model.clearAt(), tr("1 hour"));
+            QCOMPARE(model.clearAtDisplayString(), QStringLiteral("1 hour(s)"));
         }
     }
 
@@ -661,7 +662,7 @@ private slots:
         OCC::UserStatusSelectorModel model(fakeUserStatusJob);
 
         QCOMPARE(model.errorMessage(),
-            tr("Could not fetch predefined statuses. Make sure you are connected to the server."));
+            QStringLiteral("Could not fetch predefined statuses. Make sure you are connected to the server."));
     }
 
     void testError_couldNotFetchUserStatus_emitError()
@@ -671,7 +672,7 @@ private slots:
         OCC::UserStatusSelectorModel model(fakeUserStatusJob);
 
         QCOMPARE(model.errorMessage(),
-            tr("Could not fetch user status. Make sure you are connected to the server."));
+            QStringLiteral("Could not fetch status. Make sure you are connected to the server."));
     }
 
     void testError_userStatusNotSupported_emitError()
@@ -681,7 +682,7 @@ private slots:
         OCC::UserStatusSelectorModel model(fakeUserStatusJob);
 
         QCOMPARE(model.errorMessage(),
-            tr("User status feature is not supported. You will not be able to set your user status."));
+            QStringLiteral("Status feature is not supported. You will not be able to set your status."));
     }
 
     void testError_couldSetUserStatus_emitError()
@@ -692,7 +693,7 @@ private slots:
         model.setUserStatus();
 
         QCOMPARE(model.errorMessage(),
-            tr("Could not set user status. Make sure you are connected to the server."));
+            QStringLiteral("Could not set status. Make sure you are connected to the server."));
     }
 
     void testError_emojisNotSupported_emitError()
@@ -702,7 +703,7 @@ private slots:
         OCC::UserStatusSelectorModel model(fakeUserStatusJob);
 
         QCOMPARE(model.errorMessage(),
-            tr("Emojis feature is not supported. Some user status functionality may not work."));
+            QStringLiteral("Emojis are not supported. Some status functionality may not work."));
     }
 
     void testError_couldNotClearMessage_emitError()
@@ -713,7 +714,7 @@ private slots:
         model.clearUserStatus();
 
         QCOMPARE(model.errorMessage(),
-            tr("Could not clear user status message. Make sure you are connected to the server."));
+            QStringLiteral("Could not clear status message. Make sure you are connected to the server."));
     }
 
     void testError_setUserStatus_clearErrorMessage()
