@@ -1,8 +1,12 @@
 /*
-   This software is in the public domain, furnished "as is", without technical
-   support, and with no warranty, express or implied, as to its usefulness for
-   any purpose.
-*/
+ * SPDX-FileCopyrightText: 2018 Nextcloud GmbH and Nextcloud contributors
+ * SPDX-FileCopyrightText: 2013 ownCloud, Inc.
+ * SPDX-License-Identifier: CC0-1.0
+ * 
+ * This software is in the public domain, furnished "as is", without technical
+ * support, and with no warranty, express or implied, as to its usefulness for
+ * any purpose.
+ */
 
 #include <QtTest>
 #include <QTemporaryDir>
@@ -39,29 +43,29 @@ private slots:
     void testOctetsToString()
     {
         QLocale::setDefault(QLocale("en"));
-        QCOMPARE(octetsToString(999) , QString("999 B"));
-        QCOMPARE(octetsToString(1024) , QString("1 KB"));
-        QCOMPARE(octetsToString(1110) , QString("1 KB"));
-        QCOMPARE(octetsToString(1364) , QString("1 KB"));
+        QCOMPARE(octetsToString(999) , QStringLiteral("999 B"));
+        QCOMPARE(octetsToString(1024) , QStringLiteral("1 KB"));
+        QCOMPARE(octetsToString(1110) , QStringLiteral("1 KB"));
+        QCOMPARE(octetsToString(1364) , QStringLiteral("1 KB"));
 
-        QCOMPARE(octetsToString(9110) , QString("9 KB"));
-        QCOMPARE(octetsToString(9910) , QString("10 KB"));
-        QCOMPARE(octetsToString(9999) , QString("10 KB"));
-        QCOMPARE(octetsToString(10240) , QString("10 KB"));
+        QCOMPARE(octetsToString(9110) , QStringLiteral("9 KB"));
+        QCOMPARE(octetsToString(9910) , QStringLiteral("10 KB"));
+        QCOMPARE(octetsToString(9999) , QStringLiteral("10 KB"));
+        QCOMPARE(octetsToString(10240) , QStringLiteral("10 KB"));
 
-        QCOMPARE(octetsToString(123456) , QString("121 KB"));
-        QCOMPARE(octetsToString(1234567) , QString("1.2 MB"));
-        QCOMPARE(octetsToString(12345678) , QString("12 MB"));
-        QCOMPARE(octetsToString(123456789) , QString("118 MB"));
-        QCOMPARE(octetsToString(1000LL*1000*1000 * 5) , QString("4.7 GB"));
+        QCOMPARE(octetsToString(123456) , QStringLiteral("121 KB"));
+        QCOMPARE(octetsToString(1234567) , QStringLiteral("1.2 MB"));
+        QCOMPARE(octetsToString(12345678) , QStringLiteral("12 MB"));
+        QCOMPARE(octetsToString(123456789) , QStringLiteral("118 MB"));
+        QCOMPARE(octetsToString(1000LL*1000*1000 * 5) , QStringLiteral("4.7 GB"));
 
-        QCOMPARE(octetsToString(1), QString("1 B"));
-        QCOMPARE(octetsToString(2), QString("2 B"));
-        QCOMPARE(octetsToString(1024), QString("1 KB"));
-        QCOMPARE(octetsToString(1024*1024), QString("1 MB"));
-        QCOMPARE(octetsToString(1024LL*1024*1024), QString("1 GB"));
-        QCOMPARE(octetsToString(1024LL*1024*1024*1024), QString("1 TB"));
-        QCOMPARE(octetsToString(1024LL*1024*1024*1024 * 5), QString("5 TB"));
+        QCOMPARE(octetsToString(1), QStringLiteral("1 B"));
+        QCOMPARE(octetsToString(2), QStringLiteral("2 B"));
+        QCOMPARE(octetsToString(1024), QStringLiteral("1 KB"));
+        QCOMPARE(octetsToString(1024*1024), QStringLiteral("1 MB"));
+        QCOMPARE(octetsToString(1024LL*1024*1024), QStringLiteral("1 GB"));
+        QCOMPARE(octetsToString(1024LL*1024*1024*1024), QStringLiteral("1 TB"));
+        QCOMPARE(octetsToString(1024LL*1024*1024*1024 * 5), QStringLiteral("5 TB"));
     }
 
     void testLaunchOnStartup()
@@ -88,35 +92,35 @@ private slots:
 
         QDateTime current = QDateTime::currentDateTimeUtc();
 
-        QCOMPARE(durationToDescriptiveString2(0), QString("0 second(s)") );
-        QCOMPARE(durationToDescriptiveString2(5), QString("0 second(s)") );
-        QCOMPARE(durationToDescriptiveString2(1000), QString("1 second(s)") );
-        QCOMPARE(durationToDescriptiveString2(1005), QString("1 second(s)") );
-        QCOMPARE(durationToDescriptiveString2(56123), QString("56 second(s)") );
-        QCOMPARE(durationToDescriptiveString2(90*sec), QString("1 minute(s) 30 second(s)") );
-        QCOMPARE(durationToDescriptiveString2(3*hour), QString("3 hour(s)") );
-        QCOMPARE(durationToDescriptiveString2(3*hour + 20*sec), QString("3 hour(s)") );
-        QCOMPARE(durationToDescriptiveString2(3*hour + 70*sec), QString("3 hour(s) 1 minute(s)") );
-        QCOMPARE(durationToDescriptiveString2(3*hour + 100*sec), QString("3 hour(s) 2 minute(s)") );
+        QCOMPARE(durationToDescriptiveString2(0), QStringLiteral("0 second(s)") );
+        QCOMPARE(durationToDescriptiveString2(5), QStringLiteral("0 second(s)") );
+        QCOMPARE(durationToDescriptiveString2(1000), QStringLiteral("1 second(s)") );
+        QCOMPARE(durationToDescriptiveString2(1005), QStringLiteral("1 second(s)") );
+        QCOMPARE(durationToDescriptiveString2(56123), QStringLiteral("56 second(s)") );
+        QCOMPARE(durationToDescriptiveString2(90*sec), QStringLiteral("1 minute(s) 30 second(s)") );
+        QCOMPARE(durationToDescriptiveString2(3*hour), QStringLiteral("3 hour(s)") );
+        QCOMPARE(durationToDescriptiveString2(3*hour + 20*sec), QStringLiteral("3 hour(s)") );
+        QCOMPARE(durationToDescriptiveString2(3*hour + 70*sec), QStringLiteral("3 hour(s) 1 minute(s)") );
+        QCOMPARE(durationToDescriptiveString2(3*hour + 100*sec), QStringLiteral("3 hour(s) 2 minute(s)") );
         QCOMPARE(durationToDescriptiveString2(current.msecsTo(current.addYears(4).addMonths(5).addDays(2).addSecs(23*60*60))),
-                 QString("4 year(s) 5 month(s)") );
+                 QStringLiteral("4 year(s) 5 month(s)") );
         QCOMPARE(durationToDescriptiveString2(current.msecsTo(current.addDays(2).addSecs(23*60*60))),
-                 QString("2 day(s) 23 hour(s)") );
+                 QStringLiteral("2 day(s) 23 hour(s)") );
 
-        QCOMPARE(durationToDescriptiveString1(0), QString("0 second(s)") );
-        QCOMPARE(durationToDescriptiveString1(5), QString("0 second(s)") );
-        QCOMPARE(durationToDescriptiveString1(1000), QString("1 second(s)") );
-        QCOMPARE(durationToDescriptiveString1(1005), QString("1 second(s)") );
-        QCOMPARE(durationToDescriptiveString1(56123), QString("56 second(s)") );
-        QCOMPARE(durationToDescriptiveString1(90*sec), QString("2 minute(s)") );
-        QCOMPARE(durationToDescriptiveString1(3*hour), QString("3 hour(s)") );
-        QCOMPARE(durationToDescriptiveString1(3*hour + 20*sec), QString("3 hour(s)") );
-        QCOMPARE(durationToDescriptiveString1(3*hour + 70*sec), QString("3 hour(s)") );
-        QCOMPARE(durationToDescriptiveString1(3*hour + 100*sec), QString("3 hour(s)") );
+        QCOMPARE(durationToDescriptiveString1(0), QStringLiteral("0 second(s)") );
+        QCOMPARE(durationToDescriptiveString1(5), QStringLiteral("0 second(s)") );
+        QCOMPARE(durationToDescriptiveString1(1000), QStringLiteral("1 second(s)") );
+        QCOMPARE(durationToDescriptiveString1(1005), QStringLiteral("1 second(s)") );
+        QCOMPARE(durationToDescriptiveString1(56123), QStringLiteral("56 second(s)") );
+        QCOMPARE(durationToDescriptiveString1(90*sec), QStringLiteral("2 minute(s)") );
+        QCOMPARE(durationToDescriptiveString1(3*hour), QStringLiteral("3 hour(s)") );
+        QCOMPARE(durationToDescriptiveString1(3*hour + 20*sec), QStringLiteral("3 hour(s)") );
+        QCOMPARE(durationToDescriptiveString1(3*hour + 70*sec), QStringLiteral("3 hour(s)") );
+        QCOMPARE(durationToDescriptiveString1(3*hour + 100*sec), QStringLiteral("3 hour(s)") );
         QCOMPARE(durationToDescriptiveString1(current.msecsTo(current.addYears(4).addMonths(5).addDays(2).addSecs(23*60*60))),
-                 QString("4 year(s)") );
+                 QStringLiteral("4 year(s)") );
         QCOMPARE(durationToDescriptiveString1(current.msecsTo(current.addDays(2).addSecs(23*60*60))),
-                 QString("3 day(s)") );
+                 QStringLiteral("3 day(s)") );
 
     }
 
